import io
import unittest

from svgelements import *


class TestElementText(unittest.TestCase):

    def test_issue_157(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<svg>
  <g id="layer1">
    <text
       style="font-size:18px;line-height:1.25;font-family:sans-serif;stroke-width:0.264583"
       id="textobject"><tspan
         id="tspanobject"
         x="0"
         y="0">Test</tspan></text>
  </g>
</svg>
        ''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertIsNotNone(q[1].id) # Group
        self.assertIsNotNone(q[2].id) # Text
        self.assertIsNotNone(q[3].id) # TSpan



import unittest
from random import *

from svgelements import *


def get_random_cubic_bezier():
    return CubicBezier((random() * 50, random() * 50), (random() * 50, random() * 50),
                       (random() * 50, random() * 50), (random() * 50, random() * 50))


class TestElementCubicBezierLength(unittest.TestCase):

    def test_cubic_bezier_length(self):
        n = 100
        error = 0
        for _ in range(n):
            b = get_random_cubic_bezier()
            l1 = b._length_scipy()
            l2 = b._length_default(error=1e-6)
            c = abs(l1 - l2)
            error += c
            self.assertAlmostEqual(l1, l2, places=1)
        print("Average cubic-line error: %g" % (error / n))


class TestElementCubicBezierPoint(unittest.TestCase):

    def test_cubic_bezier_point_start_stop(self):
        import numpy as np
        for _ in range(1000):
            b = get_random_cubic_bezier()
            self.assertEqual(b.start, b.point(0))
            self.assertEqual(b.end, b.point(1))
            self.assertTrue(np.all(np.array([list(b.start), list(b.end)])
                                   == b.npoint([0, 1])))

    def test_cubic_bezier_point_implementations_match(self):
        import numpy as np
        for _ in range(1000):
            b = get_random_cubic_bezier()

            pos = np.linspace(0, 1, 100)

            v1 = b.npoint(pos)
            v2 = []
            for i in range(len(pos)):
                v2.append(b.point(pos[i]))

            for p, p1, p2 in zip(pos, v1, v2):
                self.assertEqual(b.point(p), Point(p1))
                self.assertEqual(Point(p1), Point(p2))



import io
import unittest

from svgelements import *


class TestParser(unittest.TestCase):

    def test_svg_examples(self):
        """Examples from the SVG spec"""
        path1 = Path('M 100 100 L 300 100 L 200 300 z')
        self.assertEqual(path1, Path(Move(100 + 100j),
                                     Line(100 + 100j, 300 + 100j),
                                     Line(300 + 100j, 200 + 300j),
                                     Close(200 + 300j, 100 + 100j)))  # CHANGED. Closed object
        self.assertTrue(path1.closed)

        # for Z command behavior when there is multiple subpaths
        path1 = Path('M 0 0 L 50 20 M 100 100 L 300 100 L 200 300 z')
        self.assertEqual(path1, Path(
            Move(0j),
            Line(0 + 0j, 50 + 20j),
            Move(50 + 20j, 100+100j),  # CHANGED. Move knows start position.
            Line(100 + 100j, 300 + 100j),
            Line(300 + 100j, 200 + 300j),
            Close(200 + 300j, 100 + 100j)))  # CHANGED. Closed object

        path1 = Path('M 100 100 L 200 200')
        path2 = Path('M100 100L200 200')
        self.assertEqual(path1, path2)

        path1 = Path('M 100 200 L 200 100 L -100 -200')
        path2 = Path('M 100 200 L 200 100 -100 -200')
        self.assertEqual(path1, path2)

        path1 = Path("""M100,200 C100,100 250,100 250,200
                              S400,300 400,200""")
        self.assertEqual(path1,
                         Path(Move(100 + 200j),
                              CubicBezier(100 + 200j, 100 + 100j, 250 + 100j, 250 + 200j),
                              CubicBezier(250 + 200j, 250 + 300j, 400 + 300j, 400 + 200j)))

        path1 = Path('M100,200 C100,100 400,100 400,200')
        self.assertEqual(path1,
                         Path(Move(100 + 200j),
                              CubicBezier(100 + 200j, 100 + 100j, 400 + 100j, 400 + 200j)))

        path1 = Path('M100,500 C25,400 475,400 400,500')
        self.assertEqual(path1,
                         Path(Move(100 + 500j),
                              CubicBezier(100 + 500j, 25 + 400j, 475 + 400j, 400 + 500j)))

        path1 = Path('M100,800 C175,700 325,700 400,800')
        self.assertEqual(path1,
                         Path(Move(100+800j),
                              CubicBezier(100 + 800j, 175 + 700j, 325 + 700j, 400 + 800j)))

        path1 = Path('M600,200 C675,100 975,100 900,200')
        self.assertEqual(path1,
                         Path(Move(600 + 200j),
                              CubicBezier(600 + 200j, 675 + 100j, 975 + 100j, 900 + 200j)))

        path1 = Path('M600,500 C600,350 900,650 900,500')
        self.assertEqual(path1,
                         Path(Move(600 + 500j),
                              CubicBezier(600 + 500j, 600 + 350j, 900 + 650j, 900 + 500j)))

        path1 = Path("""M600,800 C625,700 725,700 750,800
                              S875,900 900,800""")
        self.assertEqual(path1,
                         Path(Move(600 + 800j),
                              CubicBezier(600 + 800j, 625 + 700j, 725 + 700j, 750 + 800j),
                              CubicBezier(750 + 800j, 775 + 900j, 875 + 900j, 900 + 800j)))

        path1 = Path('M200,300 Q400,50 600,300 T1000,300')
        self.assertEqual(path1,
                         Path(Move(200 + 300j),
                              QuadraticBezier(200 + 300j, 400 + 50j, 600 + 300j),
                              QuadraticBezier(600 + 300j, 800 + 550j, 1000 + 300j)))

        path1 = Path('M300,200 h-150 a150,150 0 1,0 150,-150 z')
        self.assertEqual(path1,
                         Path(Move(300 + 200j),
                              Line(300 + 200j, 150 + 200j),
                              Arc(150 + 200j, 150 + 150j, 0, 1, 0, 300 + 50j),
                              Close(300 + 50j, 300 + 200j)))   # CHANGED. Closed object

        path1 = Path('M275,175 v-150 a150,150 0 0,0 -150,150 z')
        self.assertEqual(path1,
                         Path(Move(275 + 175j),
                              Line(275 + 175j, 275 + 25j),
                              Arc(275 + 25j, 150 + 150j, 0, 0, 0, 125 + 175j),
                              Close(125 + 175j, 275 + 175j)))   # CHANGED. Closed object

        path1 = Path('M275,175 v-150 a150,150 0 0,0 -150,150 L 275,175 z')
        self.assertEqual(path1,
                         Path(Move(275 + 175j),
                              Line(275 + 175j, 275 + 25j),
                              Arc(275 + 25j, 150 + 150j, 0, 0, 0, 125 + 175j),
                              Line(125 + 175j, 275 + 175j),
                              Close(275 + 175j, 275 + 175j)))   # CHANGED. Closed object

        path1 = Path("""M600,350 l 50,-25
                              a25,25 -30 0,1 50,-25 l 50,-25
                              a25,50 -30 0,1 50,-25 l 50,-25
                              a25,75 -30 0,1 50,-25 l 50,-25
                              a25,100 -30 0,1 50,-25 l 50,-25""")
        self.assertEqual(path1,
                         Path(Move(600 + 350j),
                              Line(600 + 350j, 650 + 325j),
                              Arc(650 + 325j, 25 + 25j, -30, 0, 1, 700 + 300j),
                              Line(700 + 300j, 750 + 275j),
                              Arc(750 + 275j, 25 + 50j, -30, 0, 1, 800 + 250j),
                              Line(800 + 250j, 850 + 225j),
                              Arc(850 + 225j, 25 + 75j, -30, 0, 1, 900 + 200j),
                              Line(900 + 200j, 950 + 175j),
                              Arc(950 + 175j, 25 + 100j, -30, 0, 1, 1000 + 150j),
                              Line(1000 + 150j, 1050 + 125j)))

    def test_wc3_examples12(self):
        """
        W3C_SVG_11_TestSuite Paths
        Test using multiple coord sets to build a polybeizer, and implicit values for initial S.
        """
        parse_path = Path

        path12 = parse_path(
            """M  100 100    C  100 20   200 20   200 100   S   300 180   300 100"""
        )
        self.assertEqual(
            path12,
            Path(
                Move(end=(100 + 100j)),
                CubicBezier(
                    start=(100 + 100j),
                    control1=(100 + 20j),
                    control2=(200 + 20j),
                    end=(200 + 100j),
                ),
                CubicBezier(
                    start=(200 + 100j),
                    control1=(200 + 180j),
                    control2=(300 + 180j),
                    end=(300 + 100j),
                ),
            ),
        )

        path12 = parse_path(
            """M  100 250    S  200 200   200 250     300 300   300 250"""
        )
        self.assertEqual(
            path12,
            Path(
                Move(end=(100 + 250j)),
                CubicBezier(
                    start=(100 + 250j),
                    control1=(100 + 250j),
                    control2=(200 + 200j),
                    end=(200 + 250j),
                ),
                CubicBezier(
                    start=(200 + 250j),
                    control1=(200 + 300j),
                    control2=(300 + 300j),
                    end=(300 + 250j),
                ),
            ),
        )

    def test_wc3_examples13(self):
        """
        W3C_SVG_11_TestSuite Paths
        Test multiple coordinates for V and H.
        """
        parse_path = Path
        path13 = parse_path(
            """   M  240.00000  56.00000    H  270.00000         300.00000 320.00000 400.00000   """
        )
        self.assertEqual(
            path13,
            Path(
                Move(end=(240 + 56j)),
                Line(start=(240 + 56j), end=(270 + 56j)),
                Line(start=(270 + 56j), end=(300 + 56j)),
                Line(start=(300 + 56j), end=(320 + 56j)),
                Line(start=(320 + 56j), end=(400 + 56j)),
            ),
        )

        path13 = parse_path(
            """   M  240.00000  156.00000    V  180.00000         200.00000 260.00000 300.00000   """
        )
        self.assertEqual(
            path13,
            Path(
                Move(end=(240 + 156j)),
                Line(start=(240 + 156j), end=(240 + 180j)),
                Line(start=(240 + 180j), end=(240 + 200j)),
                Line(start=(240 + 200j), end=(240 + 260j)),
                Line(start=(240 + 260j), end=(240 + 300j)),
            ),
        )

    def test_wc3_examples14(self):
        """
        W3C_SVG_11_TestSuite Paths
        Test implicit values for moveto. If the first command is 'm' it should be taken as an absolute moveto,
        plus implicit lineto.
        """
        parse_path = Path
        path14 = parse_path(
            """   m   62.00000  56.00000    51.96152   90.00000   -103.92304         0.00000    51.96152  -90.00000   z    m    0.00000   15.00000   38.97114   67.50000   -77.91228         0.00000   38.97114  -67.50000   z  """
        )
        self.assertEqual(
            path14,
            Path(
                Move(end=(62 + 56j)),
                Line(start=(62 + 56j), end=(113.96152000000001 + 146j)),
                Line(
                    start=(113.96152000000001 + 146j), end=(10.038480000000007 + 146j)
                ),
                Line(start=(10.038480000000007 + 146j), end=(62.00000000000001 + 56j)),
                Close(start=(62.00000000000001 + 56j), end=(62 + 56j)),
                Move(start=Point(62,56), end=(62 + 71j)),
                Line(start=(62 + 71j), end=(100.97113999999999 + 138.5j)),
                Line(
                    start=(100.97113999999999 + 138.5j),
                    end=(23.058859999999996 + 138.5j),
                ),
                Line(
                    start=(23.058859999999996 + 138.5j), end=(62.029999999999994 + 71j)
                ),
                Close(start=(62.029999999999994 + 71j), end=(62 + 71j)),
            ),
        )
        path14 = parse_path(
            """   M  177.00000   56.00000    228.96152         146.00000   125.03848  146.00000    177.00000   56.00000   Z    M  177.00000  71.00000   215.97114         138.50000   138.02886  138.50000   177.00000  71.00000   Z  """
        )

        self.assertEqual(
            path14,
            Path(
                Move(end=(177 + 56j)),
                Line(start=(177 + 56j), end=(228.96152 + 146j)),
                Line(start=(228.96152 + 146j), end=(125.03848 + 146j)),
                Line(start=(125.03848 + 146j), end=(177 + 56j)),
                Close(start=(177 + 56j), end=(177 + 56j)),
                Move(start=Point(177,56), end=(177 + 71j)),
                Line(start=(177 + 71j), end=(215.97114 + 138.5j)),
                Line(start=(215.97114 + 138.5j), end=(138.02886 + 138.5j)),
                Line(start=(138.02886 + 138.5j), end=(177 + 71j)),
                Close(start=(177 + 71j), end=(177 + 71j)),
            ),
        )

    def test_wc3_examples15(self):
        """
        W3C_SVG_11_TestSuite Paths
        'M' or 'm' command with more than one pair of coordinates are absolute
        if the moveto was specified with 'M' and relative if the moveto was
        specified with 'm'.
        """
        parse_path = Path
        path15 = parse_path("""M100,120 L160,220 L40,220 z""")
        self.assertEqual(
            path15,
            Path(
                Move(end=(100 + 120j)),
                Line(start=(100 + 120j), end=(160 + 220j)),
                Line(start=(160 + 220j), end=(40 + 220j)),
                Close(start=(40 + 220j), end=(100 + 120j)),
            ),
        )
        path15 = parse_path("""M350,120 L410,220 L290,220 z""")
        self.assertEqual(
            path15,
            Path(
                Move(end=(350 + 120j)),
                Line(start=(350 + 120j), end=(410 + 220j)),
                Line(start=(410 + 220j), end=(290 + 220j)),
                Close(start=(290 + 220j), end=(350 + 120j)),
            ),
        )
        path15 = parse_path("""M100,120 160,220 40,220 z""")
        self.assertEqual(
            path15,
            Path(
                Move(end=(100 + 120j)),
                Line(start=(100 + 120j), end=(160 + 220j)),
                Line(start=(160 + 220j), end=(40 + 220j)),
                Close(start=(40 + 220j), end=(100 + 120j)),
            ),
        )
        path15 = parse_path("""m350,120 60,100 -120,0 z""")
        self.assertEqual(
            path15,
            Path(
                Move(end=(350 + 120j)),
                Line(start=(350 + 120j), end=(410 + 220j)),
                Line(start=(410 + 220j), end=(290 + 220j)),
                Close(start=(290 + 220j), end=(350 + 120j)),
            ),
        )

    def test_wc3_examples17(self):
        """
        W3C_SVG_11_TestSuite Paths
        Test that the 'z' and 'Z' command have the same effect.
        """
        parse_path = Path
        path17 = parse_path("""M 50 50 L 50 150 L 150 150 L 150 50 z""")
        self.assertEqual(
            path17,
            Path(
                Move(end=(50 + 50j)),
                Line(start=(50 + 50j), end=(50 + 150j)),
                Line(start=(50 + 150j), end=(150 + 150j)),
                Line(start=(150 + 150j), end=(150 + 50j)),
                Close(start=(150 + 50j), end=(50 + 50j)),
            ),
        )
        path17 = parse_path("""M 50 50 L 50 150 L 150 150 L 150 50 Z""")
        self.assertEqual(
            path17,
            Path(
                Move(end=(50 + 50j)),
                Line(start=(50 + 50j), end=(50 + 150j)),
                Line(start=(50 + 150j), end=(150 + 150j)),
                Line(start=(150 + 150j), end=(150 + 50j)),
                Close(start=(150 + 50j), end=(50 + 50j)),
            ),
        )
        path17 = parse_path("""M 250 50 L 250 150 L 350 150 L 350 50 Z""")
        self.assertEqual(
            path17,
            Path(
                Move(end=(250 + 50j)),
                Line(start=(250 + 50j), end=(250 + 150j)),
                Line(start=(250 + 150j), end=(350 + 150j)),
                Line(start=(350 + 150j), end=(350 + 50j)),
                Close(start=(350 + 50j), end=(250 + 50j)),
            ),
        )

        path17 = parse_path("""M 250 50 L 250 150 L 350 150 L 350 50 z""")
        self.assertEqual(
            path17,
            Path(
                Move(end=(250 + 50j)),
                Line(start=(250 + 50j), end=(250 + 150j)),
                Line(start=(250 + 150j), end=(350 + 150j)),
                Line(start=(350 + 150j), end=(350 + 50j)),
                Close(start=(350 + 50j), end=(250 + 50j)),
            ),
        )

    def test_wc3_examples18(self):
        """
        W3C_SVG_11_TestSuite Paths
        The 'path' element's 'd' attribute ignores additional whitespace, newline characters, and commas,
        and BNF processing consumes as much content as possible, stopping as soon as a character that doesn't
        satisfy the production is encountered.
        """
        parse_path = Path
        path18a = parse_path("""M 20 40 H 40""")
        path18b = parse_path(
            """M 20 40
                 H 40"""
        )
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 60 H 40""")
        path18b = parse_path(
            """
                  M
                  20
                  60
                  H
                  40
                  """
        )
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 80 H40""")
        path18b = parse_path("""M       20,80          H    40""")
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 100 H 40#90""")
        path18b = parse_path("""M 20 100 H 40""")
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 120 H 40.5 0.6""")
        path18b = parse_path("""M 20 120 H 40.5.6""")
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 140 h 10 -20""")
        path18b = parse_path("""M 20 140 h 10-20""")
        self.assertEqual(path18a, path18b)
        path18a = parse_path("""M 20 160 H 40""")
        path18b = parse_path("""M 20 160 H 40#90""")
        self.assertEqual(path18a, path18b)

    def test_wc3_examples19(self):
        """
        W3C_SVG_11_TestSuite Paths
        Test that additional parameters to pathdata commands are treated as additional calls to the most recent command.
        """
        parse_path = Path
        path19a = parse_path("""M20 20 H40 H60""")
        path19b = parse_path("""M20 20 H40 60""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M20 40 h20 h20""")
        path19b = parse_path("""M20 40 h20 20""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M120 20 V40 V60""")
        path19b = parse_path("""M120 20 V40 60""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M140 20 v20 v20""")
        path19b = parse_path("""M140 20 v20 20""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M220 20 L 240 20 L260 20""")
        path19b = parse_path("""M220 20 L 240 20 260 20 """)
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M220 40 l 20 0 l 20 0""")
        path19b = parse_path("""M220 40 l 20 0 20 0""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M50 150 C50 50 200 50 200 150 C200 50 350 50 350 150""")
        path19b = parse_path("""M50 150 C50 50 200 50 200 150 200 50 350 50 350 150""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path(
            """M50, 200 c0,-100 150,-100 150,0 c0,-100 150,-100 150,0"""
        )
        path19b = parse_path(
            """M50, 200 c0,-100 150,-100 150,0 0,-100 150,-100 150,0"""
        )
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M50 250 S125 200 200 250 S275, 200 350 250""")
        path19b = parse_path("""M50 250 S125 200 200 250 275, 200 350 250""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M50 275 s75 -50 150 0 s75, -50 150 0""")
        path19b = parse_path("""M50 275 s75 -50 150 0 75, -50 150 0""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M50 300 Q 125 275 200 300 Q 275 325 350 300""")
        path19b = parse_path("""M50 300 Q 125 275 200 300 275 325 350 300""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M50 325 q 75 -25 150 0 q 75 25 150 0""")
        path19b = parse_path("""M50 325 q 75 -25 150 0 75 25 150 0""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M425 25 T 425 75 T 425 125""")
        path19b = parse_path("""M425 25 T 425 75 425 125""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M450 25 t 0 50 t 0 50""")
        path19b = parse_path("""M450 25 t 0 50 0 50""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M400,200 A25 25 0 0 0 425 150 A25 25 0 0 0 400 200""")
        path19b = parse_path("""M400,200 A25 25 0 0 0 425 150 25 25 0 0 0 400 200""")
        self.assertEqual(path19a, path19b)
        path19a = parse_path("""M400,300 a25 25 0 0 0 25 -50 a25 25 0 0 0 -25 50""")
        path19b = parse_path("""M400,300 a25 25 0 0 0 25 -50 25 25 0 0 0 -25 50""")
        self.assertEqual(path19a, path19b)

    def test_wc3_examples20(self):
        """
        W3C_SVG_11_TestSuite Paths
        Tests parsing of the elliptical arc path syntax.
        """
        parse_path = Path
        path20a = parse_path("""M120,120 h25 a25,25 0 1,0 -25,25 z""")
        path20b = parse_path("""M120,120 h25 a25,25 0 10 -25,25z""")
        self.assertEqual(path20a, path20b)
        path20a = parse_path("""M200,120 h-25 a25,25 0 1,1 25,25 z""")
        path20b = parse_path("""M200,120 h-25 a25,25 0 1125,25 z""")
        self.assertEqual(path20a, path20b)
        path20a = parse_path("""M280,120 h25 a25,25 0 1,0 -25,25 z""")
        self.assertRaises(Exception, 'parse_path("""M280,120 h25 a25,25 0 6 0 -25,25 z""")')
        path20a = parse_path("""M360,120 h-25 a25,25 0 1,1 25,25 z""")
        self.assertRaises(Exception, 'parse_path("""M360,120 h-25 a25,25 0 1 -1 25,25 z""")')
        path20a = parse_path("""M120,200 h25 a25,25 0 1,1 -25,-25 z""")
        path20b = parse_path("""M120,200 h25 a25,25 0 1 1-25,-25 z""")
        self.assertEqual(path20a, path20b)
        path20a = parse_path("""M200,200 h-25 a25,25 0 1,0 25,-25 z""")
        self.assertRaises(Exception, 'parse_path("""M200,200 h-25 a25,2501 025,-25 z""")')
        path20a = parse_path("""M280,200 h25 a25,25 0 1,1 -25,-25 z""")
        self.assertRaises(Exception, 'parse_path("""M280,200 h25 a25 25 0 1 7 -25 -25 z""")')
        path20a = parse_path("""M360,200 h-25 a25,25 0 1,0 25,-25 z""")
        self.assertRaises(Exception, 'parse_path("""M360,200 h-25 a25,25 0 -1 0 25,-25 z""")')

    def test_others(self):
        # Other paths that need testing:

        # Relative moveto:
        path1 = Path('M 0 0 L 50 20 m 50 80 L 300 100 L 200 300 z')
        self.assertEqual(path1, Path(
            Move(0j),
            Line(0 + 0j, 50 + 20j),
            Move(50 + 20j, 100 + 100j),  # CHANGED. Path saves the start point if it knows it.
            Line(100 + 100j, 300 + 100j),
            Line(300 + 100j, 200 + 300j),
            Close(200 + 300j, 100 + 100j)))  # CHANGED. This is a Close object now.

        # Initial smooth and relative CubicBezier
        path1 = Path("""M100,200 s 150,-100 150,0""")
        self.assertEqual(path1,
                         Path(Move(100 + 200j),
                              CubicBezier(100 + 200j, 100 + 200j, 250 + 100j, 250 + 200j)))

        # Initial smooth and relative QuadraticBezier
        path1 = Path("""M100,200 t 150,0""")
        self.assertEqual(path1,
                         Path(Move(100 + 200j),
                              QuadraticBezier(100 + 200j, 100 + 200j, 250 + 200j)))

        # Relative QuadraticBezier
        path1 = Path("""M100,200 q 0,0 150,0""")
        self.assertEqual(path1,
                         Path(Move(100 + 200j),
                              QuadraticBezier(100 + 200j, 100 + 200j, 250 + 200j)))

    def test_negative(self):
        """You don't need spaces before a minus-sign"""
        path1 = Path('M100,200c10-5,20-10,30-20')
        path2 = Path('M 100 200 c 10 -5 20 -10 30 -20')
        self.assertEqual(path1, path2)

    def test_numbers(self):
        """Exponents and other number format cases"""
        # It can be e or E, the plus is optional, and a minimum of +/-3.4e38 must be supported.
        path1 = Path('M-3.4e38 3.4E+38L-3.4E-38,3.4e-38')
        path2 = Path(Move(-3.4e+38 +  3.4e+38j), Line(-3.4e+38 + 3.4e+38j, -3.4e-38 + 3.4e-38j))
        self.assertEqual(path1, path2)

    def test_errors(self):
        self.assertRaises(ValueError, Path, 'M 100 100 L 200 200 Z 100 200')

    def test_non_path(self):
        # It's possible in SVG to create paths that has zero length,
        # we need to handle that.

        path = Path("M10.236,100.184")
        self.assertEqual(path.d(), 'M 10.236,100.184')

    def test_issue_47(self):
        arc_path_declared = Path(Move(0 + 25j), Arc(0 + 25j, 25 + 25j, 0.0, 0, 0, 0 - 25j))
        arc_path_parsed = Path('M 0 25 A25,25 0.0 0 0 0,-25')
        arc_path_parsed_scaled = Path('M 0 25 A1,1 0.0 0 0 0,-25')
        self.assertEqual(arc_path_declared, arc_path_parsed)
        self.assertEqual(arc_path_parsed_scaled, arc_path_declared)

    def test_svg_parse(self):
        s = io.StringIO(u'''<svg><path d="M0,0 L1,0 z"/></svg>''')
        svg = SVG.parse(s)
        for e in svg.elements():
            if isinstance(e, Path):
                self.assertEqual(e, "M0,0 L1,0 z")

    def test_svg_parse_group(self):
        s = io.StringIO(u'''<svg>
                        <g transform="scale(10,10)" vector-effect="non-scaling-stroke">
                        <path d="M0,0 L1,0 z"/>
                        </g>
                        </svg>''')
        svg = SVG.parse(s)
        for e in svg.elements():
            if isinstance(e, Path):
                self.assertEqual(e, "M0,0 L10,0 z")

    def test_svg_parse_group_2(self):
        s = io.StringIO(u'''<svg><g><path d="M0,0 L1,0 z"/><path d="M0,0 L1,0 z"/></g></svg>''')
        svg = SVG.parse(s)
        for e in svg.elements():
            if isinstance(e, Path):
                self.assertEqual(e, "M0,0 L1,0 z")

    def test_solo_move(self):
        move_only = Path("M0,0")
        self.assertEqual(move_only.point(0), 0 + 0j)
        self.assertEqual(move_only.point(0.5), 0 + 0j)
        self.assertEqual(move_only.point(1), 0 + 0j)
        self.assertEqual(move_only.length(), 0)

        move_onlyz = Path("M0,0Z")
        self.assertEqual(move_onlyz.point(0), 0 + 0j)
        self.assertEqual(move_onlyz.point(0.5), 0 + 0j)
        self.assertEqual(move_onlyz.point(1), 0 + 0j)
        self.assertEqual(move_onlyz.length(), 0)

        move_2_places = Path("M0,0M1,1")
        self.assertEqual(move_2_places.point(0), 0 + 0j)
        self.assertEqual(move_2_places.point(0.49), 0 + 0j)
        self.assertEqual(move_2_places.point(0.51), 1 + 1j)
        self.assertEqual(move_2_places.point(1), 1 + 1j)
        self.assertEqual(move_2_places.length(), 0)

    def test_fill_opacity_fill_none(self):
        s = io.StringIO(u'''<svg><path d="M0,0 H10 V10 H0 z" fill-opacity="1" fill="none" /></svg>''')
        svg = SVG.parse(s)
        for e in svg.elements():
            if isinstance(e, Path):
                self.assertEqual(e, "M0,0 H10 V10 H0 z")
                self.assertEqual(e.fill, "none" )


class TestParseDisplay(unittest.TestCase):
    """
    Tests for the parsing of displayed objects within an svg for conforming to the spec. Anything with a viewbox that
    has a zero width or zero height is not rendered. Any svg with a zero height or zero width is not rendered. Anything
    with a display="none" is not rendered whether this property comes from class, style, or direct attribute. Items with
    visibility="hidden" are rendered and returned but should be hidden by the end user.
    """

    def test_svgfile(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:inline">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertTrue(isinstance(q[-1], SimpleLine))

    def test_svgfile_0_width(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:inline">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_0_height(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:inline">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_viewbox_0_height(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 0" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:inline">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_viewbox_0_width(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 0 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:inline">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_display_none_inline(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="display:none">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_display_none_attribute(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g display="none">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_display_mixed(self):
        """
        All children of a display="none" are excluded, even if they override that display.
        """
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g display="none">
                        <line display="show" x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        print(q)
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_display_none_class(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <style type="text/css">\n
                        .hide { \n
                             display:none;\n
                        }\n
                        </style>\n
                        <g class="hide">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))

    def test_svgfile_display_None_class(self):
        """
        display:None is css and not svg it is case insensitive
        """
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <style type="text/css">\n
                        .hide { \n
                             display:None;\n
                        }\n
                        </style>\n
                        <g class="hide">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertFalse(isinstance(q[-1], SimpleLine))


    def test_svgfile_visibility_hidden(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="3.0cm" height="3.0cm" viewBox="0 0 100 100" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <g style="visibility:hidden">
                        <line x1="0.0" x2="0.0" y1="0.0" y2="100"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertTrue(isinstance(q[-1], SimpleLine))  # Hidden elements still exist.


class TestParseDefUse(unittest.TestCase):
    """
    Tests for Def and Use within an svg file. These must work with the definitions used within the SVG spec. This means
    that use objects must be replaced with their pure tree forms as if they are children of the use flag in question.
    """

    def test_struct_use_01(self):
        """
        The purpose of this test is to validate proper handling of
        the use element. In particular, the test checks the proper inheritance
        of properties through the shadow tree (rather than through the document
        tree).
        """
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="100%" height="100%" viewBox="0 0 480 360" xmlns="http://www.w3.org/2000/svg" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
                        <defs>
                            <g fill="red" stroke="yellow" stroke-width="3">
                                <rect id="usedRect" width="20" height="20"/>
                                <circle id="usedCircle" cx="10" cy="10" r="10"/>
                                <ellipse id="usedEllipse" cx="10" cy="10" rx="10" ry="10"/>
                                <line id="usedLine" x1="0" y1="10" x2="20" y2="10"/>
                                <path id="usedPath" d="M 0 0 L 20 0 L 20 20 L 0 20 Z"/>
                                <polygon id="usedPolygon" points="0,0 20,0 20,20 0,20 0 0"/>
                                <polyline id="usedPolyline" points="0,0 20,0 20,20"/>
                                <g id="usedG">
                                    <rect width="10" height="20"/>
                                    <rect id="half_green" x="10" width="10" height="20" fill="rgb(0,128,0)"/>
                                </g>
                                <use id="usedUse" xlink:href="#usedRect"/>
                                <text id="usedText">Text</text>
                            </g>
                        </defs>
                        <g transform="translate(150, 25)">
                            <use xlink:href="#usedRect" fill="#0F0"/>
                            <use y="30" xlink:href="#usedCircle" fill="#0F0"/>
                            <use y="60" xlink:href="#usedEllipse" fill="#0F0"/>
                            <use y="90" xlink:href="#usedLine" stroke="#0F0" stroke-width="2"/>
                            <use y="120" xlink:href="#usedPolyline" stroke="#0F0" stroke-width="2" fill="none"/>
                            <use y="150" xlink:href="#usedPolygon" fill="#0F0"/>
                            <use y="180" xlink:href="#usedPath" fill="#0F0"/>
                            <use x="180" y="0" xlink:href="#usedG" fill="#0F0"/>
                            <use x="180" y="30" xlink:href="#usedUse" fill="#0F0"/>
                            <use y="260" xlink:href="#usedText" fill="#0F0"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        for element in q:
            try:
                ident = element.id
                fill = element.fill
                stroke = element.stroke
                if ident == "half_green":
                    self.assertEqual('#008000', fill)  # Half green rectangle within used group.
                elif ident == "usedLine":
                    self.assertEqual('#0F0', stroke)
                elif ident == "usedPolyline":
                    self.assertEqual('#0F0', stroke)
                else:
                    self.assertEqual('#0F0', fill)  # Remaining are filled green.

            except AttributeError:
                pass

    def test_struct_defs_ignored(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                            <defs>
                                <g>
                                    <rect x="100" y="100" width="100" height="100" />
                                    <circle cx="100" cy="100" r="100" />
                                </g>
                            </defs>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertEqual(len(q), 1)

    def test_struct_use_unlinked(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <use href="garbage_address"/>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertEqual(len(q), 2)

    def test_parse_conditional_issue_114(self):
        import io
        from svgelements import SVG, Path

        svg_str = io.StringIO("""<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
         width="500" height="500" viewBox="0 0 500 500" preserveAspectRatio="xMidYMid meet">
         <path d="M50 150 C50 50 200 50 200 150 C200 50 350 50 350 150 z"/>
         <path d="M350 250 C50 50 200 50 200 150 C200 50 350 50 350 150 z"/> 
        </svg>""")

        for s in SVG.parse(svg_str).elements(conditional=lambda el: isinstance(el, Path)):
            self.assertEqual(type(s), Path)



import unittest
import io

from svgelements import *


class TestSVGCSS(unittest.TestCase):

    def test_issue_103(self):
        """Testing Issue 103 css class parsing
        This test is based on an Illustrator file, where the styling relies more on CSS.
        """

        q = io.StringIO(u'''<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 80 80">
        <defs>
        <style>.cls-1,.cls-2{fill:none;stroke-miterlimit:10;}.cls-1{stroke:blue;}.cls-2{stroke:red;}</style>
        </defs>
        <g id="Layer_2" data-name="Layer 2">
        <g id="Layer_1-2" data-name="Layer 1">
        <polygon points="56.59 67.4 39.86 57.28 23.01 67.22 26.34 45.99 12.83 30.88 31.62 27.88 40.12 8.6 48.41 27.97 67.17 31.17 53.5 46.14 56.59 67.4"/>
        <circle class="cls-1" cx="40" cy="40" r="35"/>
        <circle class="cls-2" cx="40" cy="40" r="39.5"/>
        </g>
        </g>
        </svg>''')
        m = SVG.parse(q)
        poly = m[0][0][0]
        circ1 = m[0][0][1]
        circ2 = m[0][0][2]

        self.assertEqual(poly.fill, "black")
        self.assertEqual(poly.stroke, "none")

        self.assertEqual(circ1.fill, "none")
        self.assertEqual(circ1.stroke, "blue")

        self.assertEqual(circ2.fill, "none")
        self.assertEqual(circ2.stroke, "red")




import unittest
import io

from svgelements import *


class TestElementColor(unittest.TestCase):
    """These tests test the basic functions of the Color element."""

    def test_color_red(self):
        reference = Color('red')
        self.assertEqual(reference, 'red')
        self.assertEqual(reference, Color('#F00'))
        self.assertEqual(reference, Color('#FF0000'))
        self.assertEqual(reference, Color("rgb(255, 0, 0)"))
        self.assertEqual(reference, Color("rgb(100%, 0%, 0%)"))
        self.assertEqual(reference, Color("rgb(300, 0, 0)"))
        self.assertEqual(reference, Color("rgb(255, -10, 0)"))
        self.assertEqual(reference, Color("rgb(110%, 0%, 0%)"))
        self.assertEqual(reference, Color("rgba(255, 0, 0, 1)"))
        self.assertEqual(reference, Color("rgba(100%, 0%, 0%, 1)"))
        self.assertEqual(reference, Color("hsl(0, 100%, 50%)"))
        self.assertEqual(reference, Color("hsla(0, 100%, 50%, 1.0)"))
        self.assertEqual(reference, Color(0xFF0000))
        color = Color()
        color.rgb = 0xFF0000
        self.assertEqual(reference, color)
        self.assertEqual(reference, Color(rgb=0xFF0000))
        self.assertEqual(reference, Color(bgr=0x0000FF))
        self.assertEqual(reference, Color(argb=0xFFFF0000))
        self.assertEqual(reference, Color(rgba=0xFF0000FF))
        self.assertEqual(reference, Color(0xFF0000, 1.0))

    def test_color_green(self):
        reference = Color('lime')  # Lime is 255 green, green is 128 green.
        self.assertEqual(reference, 'lime')
        self.assertEqual(reference, Color('#0F0'))
        self.assertEqual(reference, Color('#00FF00'))
        self.assertEqual(reference, Color("rgb(0, 255, 0)"))
        self.assertEqual(reference, Color("rgb(0%, 100%, 0%)"))
        self.assertEqual(reference, Color("rgb(0, 300, 0)"))
        self.assertEqual(reference, Color("rgb(-10, 255, 0)"))
        self.assertEqual(reference, Color("rgb(0%, 110%, 0%)"))
        self.assertEqual(reference, Color("rgba(0, 255, 0, 1)"))
        self.assertEqual(reference, Color("rgba(0%, 100%, 0%, 1)"))
        self.assertEqual(reference, Color("hsl(120, 100%, 50%)"))
        self.assertEqual(reference, Color("hsla(120, 100%, 50%, 1.0)"))
        self.assertEqual(reference, Color(0x00FF00))
        color = Color()
        color.rgb = 0x00FF00
        self.assertEqual(reference, color)
        self.assertEqual(reference, Color(rgb=0x00FF00))
        self.assertEqual(reference, Color(bgr=0x00FF00))
        self.assertEqual(reference, Color(argb=0xFF00FF00))
        self.assertEqual(reference, Color(rgba=0x00FF00FF))
        self.assertEqual(reference, Color(0x00FF00, 1.0))

    def test_color_blue(self):
        reference = Color('blue')
        self.assertEqual(reference, 'blue')
        self.assertEqual(reference, Color('#00F'))
        self.assertEqual(reference, Color('#0000FF'))
        self.assertEqual(reference, Color("rgb(0, 0, 255)"))
        self.assertEqual(reference, Color("rgb(0%, 0%, 100%)"))
        self.assertEqual(reference, Color("rgb(0, 0, 300)"))
        self.assertEqual(reference, Color("rgb(0, -10, 255)"))
        self.assertEqual(reference, Color("rgb(0%, 0%, 110%)"))
        self.assertEqual(reference, Color("rgba(0, 0, 255, 1)"))
        self.assertEqual(reference, Color("rgb(0%, 0%, 100%)"))
        self.assertEqual(reference, Color("rgba(0%, 0%, 100%, 1)"))
        self.assertEqual(reference, Color("hsl(240, 100%, 50%)"))
        self.assertEqual(reference, Color("hsla(240, 100%, 50%, 1.0)"))
        self.assertEqual(reference, Color(0x0000FF))
        color = Color()
        color.rgb = 0x0000FF
        self.assertEqual(reference, color)
        self.assertEqual(reference, Color(rgb=0x0000FF))
        self.assertEqual(reference, Color(bgr=0xFF0000))
        self.assertEqual(reference, Color(argb=0xFF0000FF))
        self.assertEqual(reference, Color(rgba=0x0000FFFF))
        self.assertEqual(reference, Color(0x0000FF, 1.0))

    def test_color_bgr(self):
        reference = Color("#26A")
        self.assertEqual(reference, Color(bgr=0xAA6622))
        reference.bgr = 0x2468AC
        self.assertEqual(reference, Color(rgb=0xAC6824))
        self.assertEqual(reference.alpha, 0xFF)

    def test_color_red_half(self):
        half_ref = Color("rgba(100%, 0%, 0%, 0.5)")
        self.assertNotEqual(half_ref, 'red')

        color = Color('red', opacity=0.5)
        self.assertEqual(color, half_ref)

    def test_color_transparent(self):
        t0 = Color('transparent')
        t1 = Color('rgba(0,0,0,0)')
        self.assertEqual(t0, t1)
        self.assertNotEqual(t0, "black")

    def test_color_hsl(self):
        c0 = Color("hsl(0, 100%, 50%)")  # red
        self.assertAlmostEqual(c0.hue, 0)
        self.assertAlmostEqual(c0.saturation, 1, places=2)
        self.assertAlmostEqual(c0.lightness, 0.5, places=2)
        self.assertEqual(c0, "red")
        c1 = Color("hsl(120, 100%, 50%)")  # lime
        self.assertAlmostEqual(c1.hue, 120)
        self.assertAlmostEqual(c1.saturation, 1, places=2)
        self.assertAlmostEqual(c1.lightness, 0.5, places=2)
        self.assertEqual(c1, "lime")
        c2 = Color("hsl(120, 100%, 19.62%)")  # dark green
        self.assertAlmostEqual(c2.hue, 120)
        self.assertAlmostEqual(c2.saturation, 1, places=2)
        self.assertAlmostEqual(c2.lightness, 0.1962, places=2)
        self.assertEqual(c2, "dark green")
        c3 = Color("hsl(120, 73.4%, 75%)")  # light green
        self.assertAlmostEqual(c3.hue, 120)
        self.assertAlmostEqual(c3.saturation, 0.734, places=2)
        self.assertAlmostEqual(c3.lightness, 0.75, places=2)
        self.assertEqual(c3, "light green")
        c4 = Color("hsl(120, 60%, 66.67%)")  # pastel green
        self.assertAlmostEqual(c4.hue, 120)
        self.assertAlmostEqual(c4.saturation, 0.6, places=2)
        self.assertAlmostEqual(c4.lightness, 0.6667, places=2)
        self.assertEqual(c4, "#77dd77")

    def test_color_hsla(self):
        c0 = Color.parse("hsl(120, 100%, 50%)")
        c1 = Color.parse("hsla(120, 100%, 50%, 1)")
        self.assertEqual(c0, c1)
        self.assertNotEqual(c0, "black")
        t1 = Color.parse("hsla(240, 100%, 50%, 0.5)")  # semi - transparent solid blue
        t2 = Color.parse("hsla(30, 100%, 50%, 0.1)")  # very transparent solid orange
        self.assertNotEqual(t1,t2)

    def test_parse_fill_opacity(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>\n
                        <svg>
                        <rect fill-opacity="0.5" x="0" y="0" width="100" height="100"/>
                        </svg>'''
                        )
        m = list(SVG.parse(q).elements())
        r = m[1]
        self.assertAlmostEqual(r.fill.opacity, 0.5, delta=1.0/255.0)

    def test_parse_stroke_opacity(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <rect stroke-opacity="0.2" stroke="red" x="0" y="0" width="100" height="100"/>
                        </svg>
                        ''')
        m = list(SVG.parse(q).elements())
        r = m[1]
        self.assertAlmostEqual(r.stroke.opacity, 0.2, delta=1.0/255.0)

    def test_color_none(self):
        color = Color(None)
        self.assertEqual(color, SVG_VALUE_NONE)
        self.assertEqual(color.red, None)
        self.assertEqual(color.green, None)
        self.assertEqual(color.blue, None)
        self.assertEqual(color.alpha, None)
        self.assertEqual(color.opacity, None)
        self.assertEqual(color.hexa, None)
        self.assertEqual(color.hex, None)
        self.assertEqual(color.blackness, None)
        self.assertEqual(color.brightness, None)
        self.assertEqual(color.hsl, None)
        self.assertEqual(color.hue, None)
        self.assertEqual(color.saturation, None)
        self.assertEqual(color.lightness, None)
        self.assertEqual(color.luma, None)
        self.assertEqual(color.luminance, None)
        self.assertEqual(color.intensity, None)

        def set_red():
            color.red = 0
        self.assertRaises(ValueError, set_red)

        def set_green():
            color.green = 0
        self.assertRaises(ValueError, set_green)

        def set_blue():
            color.blue = 0
        self.assertRaises(ValueError, set_blue)

        def set_alpha():
            color.alpha = 0
        self.assertRaises(ValueError, set_alpha)

        def set_opacity():
            color.opacity = 1
        self.assertRaises(ValueError, set_opacity)

    def test_color_hexa(self):
        for r in range(0,255,17):
            for g in range(0, 255, 17):
                for b in range(0, 255, 17):
                    for a in range(0, 255, 17):
                        c = Color()
                        c.red = r
                        c.green = g
                        c.blue = b
                        c.alpha = a
                        hexa = c.hexa
                        c2 = Color(hexa)
                        self.assertEqual(c,c2)

    def test_color_hex(self):
        for r in range(0,255,17):
            for g in range(0, 255, 17):
                for b in range(0, 255, 17):
                    c = Color()
                    c.red = r
                    c.green = g
                    c.blue = b
                    c.alpha = 255
                    hex = c.hex
                    c2 = Color(hex)
                    self.assertEqual(c,c2)

    def test_color_components(self):
        color = Color("#E9967A80")
        color2 = Color("darksalmon", .5)
        self.assertEqual(color, color2)
        self.assertEqual(color.red, 0xE9)
        self.assertEqual(color.green, 0x96)
        self.assertEqual(color.blue, 0x7A)
        self.assertEqual(color.alpha, 0x80)
        self.assertAlmostEqual(color.opacity, 0.50196078)
        self.assertEqual(color.hex, "#e9967a80")
        self.assertAlmostEqual(color.blackness, 0.0862745098)

        color.red = 0
        self.assertEqual(color.red, 0x0)
        color.green = 0
        self.assertEqual(color.green, 0x0)
        color.blue = 0
        self.assertEqual(color.blue, 0x0)
        color.alpha = 0
        self.assertEqual(color.alpha, 0x0)
        color.opacity = 1
        self.assertEqual(color.alpha, 0xFF)

        color.lightness = .5
        self.assertEqual(color.red, 0x7F)
        self.assertEqual(color.green, 0x7F)
        self.assertEqual(color.blue, 0x7F)
        self.assertEqual(color.alpha, 0xFF)

    def test_color_distinct(self):
        c0 = Color.distinct(0)
        self.assertEqual(c0, "white")
        c1 = Color.distinct(1)
        self.assertEqual(c1, "black")
        c2 = Color.distinct(2)
        self.assertEqual(c2, "red")
        c3 = Color.distinct(3)
        self.assertEqual(c3, "lime")
        c4 = Color.distinct(4)
        self.assertEqual(c4, "blue")
        c5 = Color.distinct(5)
        self.assertEqual(c5, "yellow")
        c6 = Color.distinct(6)
        self.assertEqual(c6, "cyan")
        c7 = Color.distinct(7)
        self.assertEqual(c7, "magenta")
        c8 = Color.distinct(6767890)
        self.assertEqual(c8, "#d85c3d")



import unittest

from svgelements import *

paths = [
    'M 100,100 L 300,100 L 200,300 Z',
    'M 0,0 L 50,20 M 100,100 L 300,100 L 200,300 Z',
    'M 100,100 L 200,200',
    'M 100,200 L 200,100 L -100,-200',
    'M 100,200 C 100,100 250,100 250,200 S 400,300 400,200',
    'M 100,200 C 100,100 400,100 400,200',
    'M 100,500 C 25,400 475,400 400,500',
    'M 100,800 C 175,700 325,700 400,800',
    'M 600,200 C 675,100 975,100 900,200',
    'M 600,500 C 600,350 900,650 900,500',
    'M 600,800 C 625,700 725,700 750,800 S 875,900 900,800',
    'M 200,300 Q 400,50 600,300 T 1000,300',
    'M -3.4E+38,3.4E+38 L -3.4E-38,3.4E-38',
    'M 0,0 L 50,20 M 50,20 L 200,100 Z',
    'M 600,350 L 650,325 A 25,25 -30 0,1 700,300 L 750,275',
]


class TestGeneration(unittest.TestCase):
    """Examples from the SVG spec"""

    def test_svg_examples(self):
        for path in paths[15:]:
            self.assertEqual(Path(path).d(), path)

    def test_svg_example0(self):
        path = paths[0]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example1(self):
        path = paths[1]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example2(self):
        path = paths[2]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example3(self):
        path = paths[3]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example4(self):
        path = paths[4]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example5(self):
        path = paths[5]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example6(self):
        path = paths[6]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example7(self):
        path = paths[7]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example8(self):
        path = paths[8]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example9(self):
        path = paths[9]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example10(self):
        path = paths[10]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example11(self):
        path = paths[11]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example12(self):
        path = paths[12]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example13(self):
        path = paths[13]
        self.assertEqual(Path(path).d(), path)

    def test_svg_example14(self):
        path = paths[14]
        self.assertEqual(Path(path).d(), "M 600,350 L 650,325 A 27.9508,27.9508 -30 0,1 700,300 L 750,275")
        #  Too small arc forced increase rx,ry



import io
import unittest

from svgelements import *


class TestDescriptiveElements(unittest.TestCase):

    def test_descriptive_element(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>\n
                        <svg>
                        <title>Who?</title>
                        <desc>My Friend.</desc>
                        </svg>''')
        m = SVG.parse(q)
        q = list(m.elements())
        self.assertEqual(len(q), 3)
        self.assertEqual(q[1].title, "Who?")
        self.assertEqual(q[2].desc, "My Friend.")




import unittest
import io

from svgelements import *


class TestElementShape(unittest.TestCase):

    def test_rect_dict(self):
        values = {
            'tag': 'rect',
            'rx': "4",
            'ry': "2",
            'x': "50",
            'y': "51",
            'width': "20",
            'height': "10"
        }
        e = Rect(values)
        e2 = Rect(50, 51, 20, 10, 4, 2)
        self.assertEqual(e, e2)
        e2 *= "translate(2)"
        e3 = Rect()
        self.assertNotEqual(e, e3)

    def test_line_dict(self):
        values = {
            'tag': 'rect',
            'x1': "0",
            'y1': "0",
            'x2': "100",
            'y2': "100"
        }
        e = SimpleLine(values)
        e2 = SimpleLine(0, '0px', '100px', '100px')
        e3 = SimpleLine(0, 0, 100, 100)
        self.assertEqual(e, e2)
        self.assertEqual(e, e3)
        e4 = SimpleLine()
        self.assertNotEqual(e, e4)

    def test_ellipse_dict(self):
        values = {
            'tag': 'ellipse',
            'rx': "4.0",
            'ry': "8.0",
            'cx': "22.4",
            'cy': "33.33"
        }
        e = Ellipse(values)
        e2 = Ellipse(22.4, 33.33, 4, 8)
        self.assertEqual(e, e2)
        e3 = Ellipse()
        self.assertNotEqual(e, e3)

    def test_circle_dict(self):
        values = {
            'tag': 'circle',
            'r': "4.0",
            'cx': "22.4",
            'cy': "33.33"
        }
        e = Circle(values)
        e2 = Circle(22.4, 33.33, 4)
        self.assertEqual(e, e2)
        e3 = Circle()
        self.assertNotEqual(e, e3)
        circle_d = e.d()
        self.assertEqual(Path(circle_d),
                         'M26.4,33.33A4,4 0 0,1 22.4,37.33 A4,4 0 0,1 18.4,33.33 A4,4 0 0,1 22.4,29.33 A4,4 0 0,1 26.4,33.33Z')

    def test_polyline_dict(self):
        values = {
            'tag': 'polyline',
            'points': '0,100 50,25 50,75 100,0',
        }
        e = Polyline(values)
        e2 = Polyline(0, 100, 50, 25, 50, 75, 100, 0)
        self.assertEqual(e, e2)
        e3 = Polyline()
        self.assertNotEqual(e, e3)
        polyline_d = e.d()
        self.assertEqual(Path(polyline_d), "M 0,100 L 50,25 L 50,75 L 100,0")

    def test_polygon_dict(self):
        values = {
            'tag': 'polyline',
            'points': '0,100 50,25 50,75 100,0',
        }
        e = Polygon(values)
        e2 = Polygon(0, 100, 50, 25, 50, 75, 100, 0)
        self.assertEqual(e, e2)
        e3 = Polygon()
        self.assertNotEqual(e, e3)
        polygon_d = e.d()
        self.assertEqual(Path(polygon_d), 'M 0,100 L 50,25 L 50,75 L 100,0 Z')

    def test_circle_ellipse_equal(self):
        self.assertTrue(Ellipse(center=(0, 0), rx=10, ry=10) == Circle(center="0,0", r=10.0))

    def test_transform_circle_to_ellipse(self):
        c = Circle(center="0,0", r=10.0)
        p = c * Matrix.skew_x(Angle.degrees(50))
        p.reify()
        p = c * "translate(10,1)"
        p.reify()
        p = c * "scale(10,1)"
        p.reify()
        p = c * "rotate(10deg)"
        p.reify()
        p = c * "skewy(10)"
        p.reify()
        self.assertFalse(isinstance(Circle(), Ellipse))
        self.assertFalse(isinstance(Ellipse(), Circle))

    def test_circle_decomp(self):
        circle = Circle()
        c = Path(circle.d())
        self.assertEqual(c, "M 1,0 A 1,1 0 0,1 0,1 A 1,1 0 0,1 -1,0 A 1,1 0 0,1 0,-1 A 1,1 0 0,1 1,0 Z")
        circle *= "scale(2,1)"
        c = Path(circle.d())
        self.assertEqual(c, "M 2,0 A 2,1 0 0,1 0,1 A 2,1 0 0,1 -2,0 A 2,1 0 0,1 0,-1 A 2,1 0 0,1 2,0 Z")
        circle *= "scale(0.5,1)"
        c = Path(circle.d())
        self.assertEqual(c, "M 1,0 A 1,1 0 0,1 0,1 A 1,1 0 0,1 -1,0 A 1,1 0 0,1 0,-1 A 1,1 0 0,1 1,0 Z")

    def test_circle_implicit(self):
        shape = Circle()
        shape *= "translate(40,40) rotate(15deg) scale(2,1.5)"
        self.assertAlmostEqual(shape.implicit_rx, 2.0)
        self.assertAlmostEqual(shape.implicit_ry, 1.5)
        self.assertAlmostEqual(shape.rotation, Angle.degrees(15))
        self.assertEqual(shape.implicit_center, (40, 40))

    def test_rect_implicit(self):
        shape = Rect()
        shape *= "translate(40,40) rotate(15deg) scale(2,1.5)"
        self.assertAlmostEqual(shape.implicit_x, 40)
        self.assertAlmostEqual(shape.implicit_y, 40)
        self.assertAlmostEqual(shape.implicit_width, 2)
        self.assertAlmostEqual(shape.implicit_height, 1.5)
        self.assertAlmostEqual(shape.implicit_rx, 0)
        self.assertAlmostEqual(shape.implicit_ry, 0)
        self.assertAlmostEqual(shape.rotation, Angle.degrees(15))

    def test_line_implicit(self):
        shape = SimpleLine(0, 0, 1, 1)
        shape *= "translate(40,40) rotate(15deg) scale(2,1.5)"
        self.assertAlmostEqual(shape.implicit_x1, 40)
        self.assertAlmostEqual(shape.implicit_y1, 40)
        p = Point(1, 1) * "rotate(15deg) scale(2,1.5)"
        self.assertAlmostEqual(shape.implicit_x2, 40 + p[0])
        self.assertAlmostEqual(shape.implicit_y2, 40 + p[1])
        self.assertAlmostEqual(shape.rotation, Angle.degrees(15))

    def test_circle_equals_transformed_circle(self):
        shape1 = Circle(r=2)
        shape2 = Circle().set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_rect_equals_transformed_rect(self):
        shape1 = Rect(x=0, y=0, width=2, height=2)
        shape2 = Rect(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_rrect_equals_transformed_rrect(self):
        shape1 = Rect(0, 0, 2, 2, 1, 1)
        shape2 = Rect(0, 0, 1, 1, 0.5, 0.5).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_line_equals_transformed_line(self):
        shape1 = SimpleLine(0, 0, 2, 2)
        shape2 = SimpleLine(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_polyline_equals_transformed_polyline(self):
        shape1 = Polyline(0, 0, 2, 2)
        shape2 = Polyline(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_polygon_equals_transformed_polygon(self):
        shape1 = Polyline(0, 0, 2, 2)
        shape2 = Polyline(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)
        shape2.reify()
        self.assertEqual(shape1, shape2)

    def test_polyline_not_equal_transformed_polygon(self):
        shape1 = Polyline(0, 0, 2, 2)
        shape2 = Polygon(0, 0, 1, 1) * "scale(2)"
        self.assertNotEqual(shape1, shape2)

    def test_polyline_closed_equals_transformed_polygon(self):
        shape1 = Path(Polyline(0, 0, 2, 2)) + "z"
        shape2 = Polygon(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
        self.assertEqual(shape1, shape2)

    def test_path_plus_shape(self):
        path = Path("M 0,0 z")
        path += Rect(0, 0, 1, 1)
        self.assertEqual(path, "M0,0zM0,0h1v1h-1z")

    def test_circle_not_equal_red_circle(self):
        shape1 = Circle()
        shape2 = Circle(stroke="red")
        self.assertNotEqual(shape1, shape2)
        shape1 = Circle()
        shape2 = Circle(fill="red")
        self.assertNotEqual(shape1, shape2)

    def test_rect_initialize(self):
        shapes = (
            Rect(),
            Rect(0),
            Rect(0, 0),
            Rect(0, 0, 1),
            Rect(0, 0, 1, 1),
            Rect(0, y=0),
            Rect(0, y=0, width=1),
            Rect(0, y=0, width=1, height=1),
            Rect(width=1, height=1, x=0, y=0),
            Rect(0, 0, 1, 1, 0, 0),
            Rect(0, 0, 1, 1, rx=0, ry=0)
        )
        for s in shapes:
            self.assertEqual(shapes[0], s)

    def test_circle_initialize(self):
        shapes = (
            Circle(),
            Circle(0, 0),
            Circle(center=(0, 0), r=1),
            Circle("0px", "0px", 1),
            Ellipse("0", "0", 1, 1),
            Ellipse("0", "0", rx=1, ry=1),
            Ellipse(0, 0, 1, ry=1),
            Circle(Circle()),
            Circle({"cx": 0, "cy": 0, "r": 1}),
            Ellipse({"cx": 0, "cy": 0, "rx": 1}),
            Ellipse({"cx": 0, "cy": 0, "ry": 1}),
            Ellipse({"cx": 0, "cy": 0, "rx": 1, "ry": 1.0}),
            Circle(Ellipse()),
            Ellipse(Circle())
        )
        for s in shapes:
            self.assertEqual(shapes[0], s)

    def test_polyline_initialize(self):
        shapes = (
            Polyline(0, 0, 1, 1),
            Polyline((0, 0), (1, 1)),
            Polyline(points=((0, 0), (1, 1))),
            Polyline("0,0", "1,1"),
            Polyline("0,0", (1, 1)),
            Polyline("0,0", Point(1, 1)),
            Polyline({"points": "0,0,1,1"}),
            Polyline(Polyline(0, 0, 1, 1)),
            Path("M0,0L1,1"),
            SimpleLine(0, 0, 1, 1),
        )
        for s in shapes:
            self.assertEqual(shapes[0], s)

    def test_polygon_initialize(self):
        shapes = (
            Polygon(0, 0, 1, 1),
            Polygon((0, 0), (1, 1)),
            Polygon(points=((0, 0), (1, 1))),
            Polygon("0,0", "1,1"),
            Polygon("0,0", (1, 1)),
            Polygon("0,0", Point(1, 1)),
            Polygon({"points": "0,0,1,1"}),
            Polygon(Polyline(0, 0, 1, 1)),
            Polygon("0,0,1,1"),
            Path("M0,0L1,1z"),
        )
        for s in shapes:
            self.assertEqual(shapes[0], s)

    def test_shapes_repr(self):
        s = Rect(fill='red')
        self.assertEqual(repr(s), "Rect(width=1, height=1, fill='#ff0000')")
        s = Ellipse(fill='red')
        self.assertEqual(repr(s), "Ellipse(cx=0, cy=0, r=1, fill='#ff0000')")
        s = Circle(fill='red')
        self.assertEqual(repr(s), "Circle(cx=0, cy=0, r=1, fill='#ff0000')")
        s = SimpleLine(fill='red')
        self.assertEqual(repr(s), "SimpleLine(x1=0.0, y1=0.0, x2=0.0, y2=0.0, fill='#ff0000')")
        s = Polygon(fill='red')
        self.assertEqual(repr(s), "Polygon(points='', fill='#ff0000')")
        s = Polyline(fill='red')
        self.assertEqual(repr(s), "Polyline(points='', fill='#ff0000')")
        s = Path(fill='red')
        self.assertEqual(repr(s), "Path(fill='#ff0000')")

    def test_shape_bbox(self):
        s = Rect() * 'scale(20)'
        self.assertEqual(s.bbox(False), (0, 0, 1, 1))
        self.assertEqual(s.bbox(True), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(False), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(True), (0, 0, 1, 1))
        s = Circle() * 'scale(20)'
        self.assertEqual(s.bbox(False), (-1, -1, 1, 1))
        self.assertEqual(s.bbox(True), (-20, -20, 20, 20))
        self.assertNotEqual(s.bbox(False), (-20, -20, 20, 20))
        self.assertNotEqual(s.bbox(True), (-1, -1, 1, 1))
        s = Ellipse() * 'scale(20)'
        self.assertEqual(s.bbox(False), (-1, -1, 1, 1))
        self.assertEqual(s.bbox(True), (-20, -20, 20, 20))
        self.assertNotEqual(s.bbox(False), (-20, -20, 20, 20))
        self.assertNotEqual(s.bbox(True), (-1, -1, 1, 1))
        s = Polygon() * 'scale(20)'
        self.assertEqual(s.bbox(False), None)
        self.assertEqual(s.bbox(True), None)
        self.assertNotEqual(s.bbox(False), (0, 0, 0, 0))
        self.assertNotEqual(s.bbox(True), (0, 0, 0, 0))
        s = Polyline() * 'scale(20)'
        self.assertEqual(s.bbox(False), None)
        self.assertEqual(s.bbox(True), None)
        self.assertNotEqual(s.bbox(False), (0, 0, 0, 0))
        self.assertNotEqual(s.bbox(True), (0, 0, 0, 0))
        s = Polygon("0,0 0,1 1,1 1,0 0,0") * 'scale(20)'
        self.assertEqual(s.bbox(False), (0, 0, 1, 1))
        self.assertEqual(s.bbox(True), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(False), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(True), (0, 0, 1, 1))
        s = Polyline("0,0 0,1 1,1 1,0 0,0") * 'scale(20)'
        self.assertEqual(s.bbox(False), (0, 0, 1, 1))
        self.assertEqual(s.bbox(True), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(False), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(True), (0, 0, 1, 1))
        s = SimpleLine(0, 0, 1, 1) * 'scale(20)'
        self.assertEqual(s.bbox(False), (0, 0, 1, 1))
        self.assertEqual(s.bbox(True), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(False), (0, 0, 20, 20))
        self.assertNotEqual(s.bbox(True), (0, 0, 1, 1))

    def test_rect_rot_equal_rect_path_rotate(self):
        r = Rect(10, 10, 8, 4)
        a = r.d()
        b = Path(a).d()
        self.assertEqual(a, b)
        a = (Path(r.d()) * "rotate(0.5turns)").d()
        b = (r * "rotate(0.5turns)").d()
        self.assertEqual(a, b)

    def test_rect_reify(self):
        """Reifying a rotated rect."""
        reification_checks(self, Rect())
        reification_checks(self, Rect(2, 2, 4, 4))

        shape = Rect() * "rotate(-90) translate(20,0)"
        t = Rect(0, -20, 1, 1)
        t *= "rotate(-90, 0, -20)"
        self.assertEqual(t, shape)

    def test_circle_reify(self):
        """Reifying a rotated circle."""
        reification_checks(self, Circle())
        reification_checks(self, Circle(2, 2, 4, 4))

    def test_ellipse_reify(self):
        """Reifying a rotated ellipse."""
        reification_checks(self, Ellipse(rx=1, ry=2))
        reification_checks(self, Ellipse(2, 2, 5, 8))

    def test_polyline_reify(self):
        """Reifying a rotated polyline."""
        reification_checks(self, Polyline("0,0 1,1 2,2"))
        reification_checks(self, Polyline("0,0 1,1 2,0"))

    def test_polygon_reify(self):
        """Reifying a rotated polygon."""
        reification_checks(self, Polygon("0,0 1,1 2,2"))
        reification_checks(self, Polygon("0,0 1,1 2,0"))

    def test_line_reify(self):
        """Reifying a rotated line."""
        reification_checks(self, SimpleLine(0, 0, 1, 1))
        reification_checks(self, SimpleLine(2, 2, 1, 0))

    def test_path_reify(self):
        """Reifying a path."""
        reification_checks(self, Path("M0,0L1,1L1,0z"))
        reification_checks(self, Path("M100,100L70,70L45,0z"))

    def test_shapes_degenerate(self):
        """Testing Degenerate Shapes"""
        self.assertEqual(Rect(0, 0, 0, 100).d(), '')
        self.assertEqual(Rect(0, 0, 100, 0).d(), '')
        self.assertEqual(Circle(0, 0, 0).d(), '')
        self.assertEqual(Ellipse(0,0,0,100).d(), '')
        self.assertEqual(Ellipse(0, 0, 100, 0).d(), '')
        self.assertEqual(Polygon(points='').d(), '')

    def test_issue_95(self):
        """Testing Issue 95 stroke-width"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <ellipse style="stroke:#fc0000;stroke-width:1;fill:none" cx="0" cy="0" rx="1" ry="1" transform="scale(100) rotate(-90,0,0)"/>
                        <rect style="stroke:#fc0000;stroke-width:1;fill:none" x="0" y="0" width="10" height="10" transform="scale(100) rotate(-90,0,0)"/>
                        </svg>''')
        m = SVG.parse(q)
        ellipse = m[0]
        for i in range(5):
            ellipse = ellipse.reify()
        self.assertEqual(ellipse.stroke_width, 1.0)

        rect = m[1]
        for i in range(5):
            rect = rect.reify()
        self.assertEqual(rect.stroke_width, 1.0)

    def test_issue_99(self):
        """Test Issue of inverted circle reified location"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg
                            width="82.475mm"
                            height="35.215mm"
                            viewBox="24.766026 -242.607513 82.475082 35.214996"
                            version="1.1"
                        >
                        <circle
                            transform="scale(1,-1)"
                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
                            r="2"
                            cx="100.41245"
                            cy="211.59723"
                            id="circle2" /></svg>
        ''')
        m = SVG.parse(q, reify=False)
        q = copy(m[0])
        r = copy(m[0])
        self.assertEqual(q, r)
        q.reify()
        r = Path(r)
        q = Path(q)
        self.assertEqual(q, r)
        r.reify()
        q.reify()
        self.assertEqual(q, r)

    def test_issue_99b(self):
        """Test Issue of double inverted circle reified location"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg
                            width="82.475mm"
                            height="35.215mm"
                            viewBox="24.766026 -242.607513 82.475082 35.214996"
                            version="1.1"
                        >
                        <circle
                            transform="scale(-1,-1)"
                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
                            r="2"
                            cx="100.41245"
                            cy="211.59723"
                            id="circle2" /></svg>
        ''')
        m = SVG.parse(q, reify=False)
        q = copy(m[0])
        r = copy(m[0])
        self.assertEqual(q, r)
        q.reify()
        r = Path(r)
        q = Path(q)
        self.assertEqual(q, r)
        r.reify()
        q.reify()
        self.assertEqual(q, r)

    def test_issue_99c(self):
        """Test Issue of inverted rect reified location"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg
                            width="82.475mm"
                            height="35.215mm"
                            viewBox="24.766026 -242.607513 82.475082 35.214996"
                            version="1.1"
                        >
                        <rect
                            transform="scale(1,-1)"
                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
                            rx="2"
                            x="100.41245"
                            y="211.59723"
                            width="100"
                            height="100"
                            id="circle2" /></svg>
        ''')
        m = SVG.parse(q, reify=False)
        q = copy(m[0])
        r = copy(m[0])
        self.assertEqual(q, r)
        q.reify()
        r = Path(r)
        q = Path(q)
        self.assertEqual(q, r)
        r.reify()
        q.reify()
        self.assertEqual(q, r)

    def test_issue_99d(self):
        """Test Issue of double inverted rect reified location"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg
                            width="82.475mm"
                            height="35.215mm"
                            viewBox="24.766026 -242.607513 82.475082 35.214996"
                            version="1.1"
                        >
                        <rect
                            transform="scale(-1,-1)"
                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
                            rx="2"
                            x="100.41245"
                            y="211.59723"
                            width="100"
                            height="100"
                            id="circle2" /></svg>
        ''')
        m = SVG.parse(q, reify=False)
        q = copy(m[0])
        r = copy(m[0])
        self.assertEqual(q, r)
        q.reify()
        r = Path(r)
        q = Path(q)
        self.assertEqual(q, r)
        r.reify()
        q.reify()
        self.assertEqual(q, r)

    def test_issue_104(self):
        """Testing Issue 104 degenerate parsing"""
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <polygon points=""/>
                        <polygon/>
                        <rect x="0" y="0" width="0" height="10"/>
                        <circle cx="0" cy="0" r="0"/>
                        </svg>''')
        m = SVG.parse(q)
        self.assertEqual(len(m), 0)

    def test_rect_strict(self):
        values = {
            'tag': 'rect',
            'rx': "-4",
            'x': "50",
            'y': "51",
            'width': "20",
            'height': "10"
        }
        e = Rect(values)
        e2 = Rect(50, 51, 20, 10)
        self.assertEqual(e, e2)

        e3 = Rect(values)
        e3._strict = False  # unstrict rx-negative rectangles, have scooped corners.
        self.assertNotEqual(e3, e2)

        values['ry'] = 4
        e4 = Rect(values)
        self.assertEqual(e, e4)

    def test_shape_npoints(self):
        import numpy as np
        shapes = [
            Rect(10, 20, 300, 340),
            Circle(10, 10, 5),
            Ellipse(50, 50, 30, 20),
            Polygon(points=((10, 10), (20, 30), (50, 20))),
            Polyline(points=((10, 10), (20, 30), (50, 20), (100, 120))),
        ]

        for shape in shapes:
            pos = np.linspace(0, 1, 1000)
            pts1 = shape.npoint(pos)  # Test rendered worthless.
            v2 = []
            for i in range(len(pos)):
                v2.append(shape.point(pos[i]))

            for p, p1, p2 in zip(pos, pts1, v2):
                self.assertEqual(shape.point(p), Point(p1))
                self.assertEqual(Point(p1), Point(p2))


def reification_checks(test, shape):
    correct_reify(test, shape * "rotate(-90) translate(20,0)")
    correct_reify(test, shape * "rotate(12turn)")
    correct_reify(test, shape * "translate(20,0)")
    correct_reify(test, shape * "scale(2) translate(20,0)")
    correct_reify(test, shape * "rotate(90) scale(-1) translate(20,0)")
    correct_reify(test, shape * "rotate(90) translate(20,0)")
    correct_reify(test, shape * "skewX(10)")
    correct_reify(test, shape * "skewY(10)")


def correct_reify(test, shape):
    path = abs(Path(shape))
    reified = abs(copy(shape))
    test.assertEqual(path, shape)
    test.assertEqual(reified, shape)
    test.assertEqual(reified, path)



import unittest

from svgelements import *


class TestPath(unittest.TestCase):
    """Tests of dunder methods of the SVG Path element."""

    def test_path_iadd_str(self):
        p1 = Path("M0,0")
        p1 += "z"
        self.assertEqual(p1, Path("M0,0z"))

        p1 = Path("M2,2z")
        p1 += "M1,1z"
        p1 += "M0,0z"
        subpaths = list(p1.as_subpaths())
        self.assertEqual(str(subpaths[0]), Path("M2,2z"))
        self.assertEqual(str(subpaths[1]), Path("M1,1z"))
        self.assertEqual(str(subpaths[2]), Path("M0,0z"))

    def test_path_add_str(self):
        p1 = Path("M0,0")
        p2 = p1 + "z"
        p1 += "z"
        self.assertEqual(p1, p2)

    def test_path_radd_str(self):
        p1 = Path("M0,0z")
        p2 = "M1,1z" + p1
        subpaths = list(p2.as_subpaths())
        self.assertEqual(str(subpaths[0]), str(Path("M1,1z")))
        self.assertEqual(str(subpaths[1]), str(Path("M0,0z")))

    def test_path_iadd_segment(self):
        p1 = Path("M0,0")
        p1 += Line((0, 0), (7, 7))
        p1 += "z"
        self.assertEqual(p1, Path("M0,0 L7,7 z"))

    def test_path_add_segment(self):
        p1 = Path("M0,0")
        p2 = p1 + Line((0, 0), (7, 7))
        p1 += "z"
        p2 += "z"
        self.assertEqual(p1, Path("M0,0 z"))
        self.assertEqual(p2, Path("M0,0 L7,7 z"))

    def test_path_radd_segment(self):
        p1 = Path("L7,7")
        p1 = Move((0, 0)) + p1
        p1 += "z"
        self.assertEqual(p1, Path("M0,0 L7,7 z"))

    def test_path_from_segment(self):
        p1 = Move(0) + Line(0, (7, 7)) + "z"
        self.assertEqual(p1, Path("M0,0 L7,7 z"))

        p1 = Move(0) + "L7,7" + "z"
        self.assertEqual(p1, Path("M0,0 L7,7 z"))

        p1 = Move(0) + "L7,7z"
        self.assertEqual(p1, Path("M0,0 L7,7 z"))

    def test_segment_mult_string(self):
        p1 = Move(0) * "translate(200,200)"
        self.assertEqual(p1, Move((200, 200)))

        p1 += "z"
        self.assertEqual(p1, Path("M200,200z"))

    def test_path_mult_string(self):
        p1 = Path(Move(0)) * "translate(200,200)"
        self.assertEqual(p1, "M200,200")

        p1 = Path(Move(0)).set('vector-effect', 'non-scaling-stroke') * "scale(0.5) translateX(200)"
        self.assertEqual(p1, "M100,0")
        self.assertNotEqual(p1, "M200,0")

        p1 = Path(Move(0)).set('vector-effect', 'non-scaling-stroke') * "translateX(200) scale(0.5)"
        self.assertEqual(p1, "M200,0")
        self.assertNotEqual(p1, "M100,0")

    def test_path_equals_string(self):
        self.assertEqual(Path("M55,55z"), "M55,55z")
        self.assertEqual(Path("M55 55z"), "M   55,   55z")
        self.assertTrue(Move(0) * "translate(55,55)" + "z" == "m 55, 55Z")
        self.assertTrue(Move(0) * "rotate(0.50turn,100,0)" + "z" == "M200,0z")
        self.assertFalse(Path(Move(0)) == "M0,0z")
        self.assertEqual(Path("M50,50 100,100 0,100 z").set('vector-effect', 'non-scaling-stroke') * "scale(0.1)",
                         "M5,5 L10,10 0,10z")
        self.assertNotEqual(Path("M50,50 100,100 0,100 z") * "scale(0.11)", "M5,5 L10,10 0,10z")
        self.assertEqual(
            Path("M0,0 h10 v10 h-10 v-10z").set('vector-effect', 'non-scaling-stroke') * "scale(0.2) translate(-5,-5)",
            "M -1,-1, L1,-1, 1,1, -1,1, -1,-1 Z"
        )

    def test_path_mult_matrix(self):
        p = Path("L20,20 40,40") * Matrix("Rotate(20)")
        self.assertEqual(p, "L11.953449549205,25.634255282232 23.906899098410,51.268510564463")
        p.reify()
        p += "L 100, 100"
        p += Close()
        self.assertEqual(p, Path("L11.953449549205,25.634255282232 23.906899098410,51.268510564463 100,100 z"))

    def test_partial_path(self):
        p1 = Path("M0,0")
        p2 = Path("L7,7")
        p3 = Path("Z")
        q = p1 + p2 + p3
        m = Path("M0,0 7,7z")
        self.assertEqual(q, m)




import io
import unittest

from svgelements import *


class TestElementsRepr(unittest.TestCase):
    """Tests the functionality of the repr for elements."""

    def test_repr_length(self):
        obj = Length("10cm")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_color(self):
        obj = Color("red")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_point(self):
        obj = Point("20.3,3.1615926535")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_angle(self):
        obj = Angle.parse("1.1turn")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertAlmostEqual(obj, obj2)

    def test_repr_matrix(self):
        obj = Matrix("rotate(20)")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_viewbox(self):
        obj = Viewbox("0 0 100 60")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_move(self):
        obj = Move(0.1, 50)
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_close(self):
        obj = Close(0.1, 50)
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_line(self):
        obj = Line(start=(0.2, 0.99), end=(0.1, 22.9996))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Line(end=(0.1, 22.9996))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_qbez(self):
        obj = QuadraticBezier(start=(0.2, 0.99), control=(-3,-3), end=(0.1, 22.9996))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_cbez(self):
        obj = CubicBezier(start=(0.2, 0.99), control1=(-3, -3), control2=(-4, -4), end=(0.1, 22.9996))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_arc(self):
        obj = Arc(start=(0,0), end=(0,100), control=(50,50))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_path(self):
        obj = Path("M0,0Z")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Path("M0,0L100,100Z")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Path("M0,0L100,100Z", transform="scale(4)")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_rect(self):
        obj = Rect(x=100, y=100, width=500, height=500)
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Rect(x=100, y=100, width=500, height=500, transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_ellipse(self):
        obj = Ellipse(cx=100, cy=100, rx=500, ry=500)
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Ellipse(cx=100, cy=100, rx=500, ry=500, transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_circle(self):
        obj = Circle(cx=100, cy=100, r=500)
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Circle(cx=100, cy=100, r=500, transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_simpleline(self):
        obj = SimpleLine(start=(0,0), end=(100,100))
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = SimpleLine(start=(0, 0), end=(100, 100), transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_polyline(self):
        obj = Polyline("0,0 7,7 10,10 0 20")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Polyline("0,0 7,7 10,10 0 20", transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_polygon(self):
        obj = Polygon("0,0 7,7 10,10 0 20")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Polygon("0,0 7,7 10,10 0 20", transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_group(self):
        obj = Group()
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = Group(transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_clippath(self):
        obj = ClipPath()
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_pattern(self):
        obj = Pattern()
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_text(self):
        obj = SVGText(x=0, y=0, text="Hello")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

        obj = SVGText(x=0, y=0, text="Hello", transform="scale(2)", stroke="red", fill="blue")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_image(self):
        obj = SVGImage(href="test.png", transform="scale(2)")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_desc(self):
        obj = Desc("Describes Object")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)

    def test_repr_title(self):
        obj = Title(title="SVG Description")
        repr_obj = repr(obj)
        obj2 = eval(repr_obj)
        self.assertTrue(obj == obj2)
        self.assertFalse(obj != obj2)




import io
import unittest

from svgelements import *


class TestElementLength(unittest.TestCase):
    """Tests the functionality of the Length Element."""

    def test_length_parsing(self):
        self.assertAlmostEqual(Length('10cm'), (Length('100mm')))
        self.assertNotEqual(Length("1mm"), 0)
        self.assertNotEqual(Length("1cm"), 0)
        self.assertNotEqual(Length("1in"), 0)
        self.assertNotEqual(Length("1px"), 0)
        self.assertNotEqual(Length("1pc"), 0)
        self.assertNotEqual(Length("1pt"), 0)
        self.assertNotEqual(Length("1%").value(relative_length=100), 0)
        self.assertEqual(Length("50%").value(relative_length=100), 50.0)

    def test_distance_matrix(self):
        m = Matrix("Translate(20mm,50%)", ppi=1000, width=600, height=800)
        self.assertEqual(Matrix(1, 0, 0, 1, 787.402, 400), m)
        m = Matrix("Translate(20mm,50%)")
        m.render(ppi=1000, width=600, height=800)
        self.assertEqual(Matrix(1, 0, 0, 1, 787.402, 400), m)

    def test_rect_distance_percent(self):
        rect = Rect("0%", "0%", "100%", "100%")
        rect.render(relative_length="1mm", ppi=DEFAULT_PPI)
        self.assertEqual(rect, Path("M 0,0 H 3.7795296 V 3.7795296 H 0 z"))
        rect = Rect("0%", "0%", "100%", "100%")
        rect.render(relative_length="1in", ppi=DEFAULT_PPI)
        self.assertEqual(rect, Path("M 0,0 H 96 V 96 H 0 z"))

    def test_circle_distance_percent(self):
        shape = Circle(0, 0, "50%")
        shape.render(relative_length="1in", ppi=DEFAULT_PPI)
        print(shape.d())
        self.assertEqual(
            shape,
            Path('M48,0A48,48 0 0,1 0,48A48,48 0 0,1-48,0A48,48 0 0,1 0,-48A48,48 0 0,1 48,0Z')
        )

    def test_length_division(self):
        self.assertEqual(Length("1mm") // Length('1mm'), 1.0)
        self.assertEqual(Length("1mm") / Length('1mm'), 1.0)
        self.assertEqual(Length('1in') / '1in', 1.0)
        self.assertEqual(Length('1cm') / '1mm', 10.0)

    def test_length_compare(self):
        self.assertTrue(Length('1in') < Length('2.6cm'))
        self.assertTrue(Length('1in') < '2.6cm')
        self.assertFalse(Length('1in') < '2.5cm')
        self.assertTrue(Length('10mm') >= '1cm')
        self.assertTrue(Length('10mm') <= '1cm')
        self.assertTrue(Length('11mm') >= '1cm')
        self.assertTrue(Length('10mm') <= '1.1cm')
        self.assertFalse(Length('11mm') <= '1cm')
        self.assertFalse(Length('10mm') >= '1.1cm')
        self.assertTrue(Length('20%') > '10%')
        self.assertRaises(ValueError, lambda: Length('20%') > '1in')
        self.assertRaises(ValueError, lambda: Length('20px') > '1in')
        self.assertRaises(ValueError, lambda: Length('20pc') > '1in')
        self.assertRaises(ValueError, lambda: Length('20em') > '1in')
        self.assertEqual(max(Length('1in'), Length('2.5cm')), '1in')

    def test_length_parsed(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <rect x="1in" y="1in" width="10in" height="10in"/>
                        </svg>''')
        m = SVG.parse(q, ppi=96.0)
        q = list(m.elements())
        self.assertEqual(q[1].x, 96.0)
        self.assertEqual(q[1].y, 96.0)
        self.assertEqual(q[1].width, 960)
        self.assertEqual(q[1].height, 960)

    def test_length_parsed_percent(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <rect x="25%" y="25%" width="50%" height="50%"/>
                        </svg>''')
        m = SVG.parse(q, width=1000, height=1000)
        q = list(m.elements())
        self.assertEqual(q[1].x, 250)
        self.assertEqual(q[1].y, 250)
        self.assertEqual(q[1].width, 500)
        self.assertEqual(q[1].height, 500)

    def test_length_parsed_percent2(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>\n
                        <svg width="1in" height="1in">
                        <rect x="25%" y="25%" width="50%" height="50%"/>
                        </svg>''')
        m = SVG.parse(q, width=1000, height=1000)
        q = list(m.elements())
        self.assertEqual(q[1].x, 24)
        self.assertEqual(q[1].y, 24)
        self.assertEqual(q[1].width, 48)
        self.assertEqual(q[1].height, 48)

    def test_length_parsed_percent3(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg width="1in" height="1in">
                        <rect x="25%" y="25%" width="50%" height="50%"/>
                        </svg>''')
        m = SVG.parse(q, width=500, height=500)
        q = list(m.elements())
        self.assertEqual(q[1].x, 24)
        self.assertEqual(q[1].y, 24)
        self.assertEqual(q[1].width, 48)
        self.assertEqual(q[1].height, 48)

    def test_length_parsed_percent4(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg viewbox="0 0 960 960" width="1in" height="1in">
                        <rect x="25%" y="25%" width="50%" height="50%"/>
                        </svg>''')
        m = SVG.parse(q, width="garbage", height=500)
        q = list(m.elements())
        self.assertEqual(q[1].x, 24)
        self.assertEqual(q[1].y, 24)
        self.assertEqual(q[1].width, 48)
        self.assertEqual(q[1].height, 48)

    def test_length_parsed_percent5(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg viewbox="0 0 960 960">
                        <rect x="25%" y="25%" width="50%" height="50%"/>
                        <rect x="240" y="240" width="480" height="480"/>
                        </svg>''')
        m = SVG.parse(q, width="1in", height="1in")
        q = list(m.elements())
        self.assertEqual(q[1].x, 24)
        self.assertEqual(q[1].y, 24)
        self.assertEqual(q[1].width, 48)
        self.assertEqual(q[1].height, 48)
        self.assertEqual(q[2].x, 240)
        self.assertEqual(q[2].y, 240)
        self.assertEqual(q[2].width, 480)
        self.assertEqual(q[2].height, 480)

    def test_length_parsed_percent6(self):
        q = io.StringIO(u'''<svg version="1.1" baseProfile="basic" id="svg-root"
                        width="100%" height="100%" viewBox="0 0 480 360"
                        xmlns="http://www.w3.org/2000/svg">
                        <g transform="translate(5, 50) scale(4)">
                        <circle cx="7.5" cy="7.5" r="2.5" fill="black"/>
                        <circle cx="1.563%" cy="2.083%" r=".3535%" fill="fuchsia"/>
                        </g>
                        <g transform="translate(30, 260)  skewX(45) scale(4)">
                        <circle cx="0" cy="0" r="3.536" fill="black"/>
                        <circle cx="10" cy="0" r="3.536px" fill="fuchsia"/>
                        <circle cx="20" cy="0" r=".8334%" fill="green"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q, width="10000", height="10000")
        q = list(m.elements())
        self.assertAlmostEqual(q[2].cx, q[3].cx, delta=1)
        self.assertAlmostEqual(q[2].cy, q[3].cy, delta=1)
        self.assertAlmostEqual(q[5].rx, q[6].rx, delta=1)
        self.assertAlmostEqual(q[6].rx, q[7].rx, delta=1)



import unittest
import io

from svgelements import *


class TestElementCopy(unittest.TestCase):
    """These tests test the validity of object copy."""

    def test_copy_objects(self):
        # CSS OBJECTS

        length = Length('1in')
        length_copy = copy(length)
        self.assertEqual(length, length_copy)

        color = Color('red')
        color_copy = copy(color)
        self.assertEqual(color, color_copy)

        point = Point("2,4.58")
        point_copy = copy(point)
        self.assertEqual(point, point_copy)

        angle = Angle.parse('1.2grad')
        angle_copy = copy(angle)
        self.assertEqual(angle, angle_copy)

        matrix = Matrix("scale(4.5) translate(2,2.4) rotate(40grad)")
        matrix_copy = copy(matrix)
        self.assertEqual(matrix, matrix_copy)

        # SVG OBJECTS
        viewbox = Viewbox('0 0 103 109', preserveAspectRatio="xMaxyMin slice")
        viewbox_copy = copy(viewbox)
        # self.assertEqual(viewbox, viewbox_copy)

        svgelement = SVGElement({'tag': "element", 'id': 'testelement1234'})
        svgelement_copy = copy(svgelement)
        self.assertIsNotNone(svgelement_copy.values)
        # self.assertEqual(svgelement, svgelement_copy)

        # PATH SEGMENTS
        move = Move((8,8.78))
        move_copy = copy(move)
        self.assertEqual(move, move_copy)

        close = Close()
        close_copy = copy(close)
        self.assertEqual(close, close_copy)

        line = Line((8, 8.78))
        line_copy = copy(line)
        self.assertEqual(line, line_copy)

        quad = QuadraticBezier((8, 8.78), (50, 50.78), (50, 5))
        quad_copy = copy(quad)
        self.assertEqual(quad, quad_copy)

        cubic = CubicBezier((8, 8.78), (1, 6.78), (8, 9.78), (50, 5))
        cubic_copy = copy(cubic)
        self.assertEqual(cubic, cubic_copy)

        arc = Arc(start=(0,0), end=(25,0), control=(10,10))
        arc_copy = copy(arc)
        self.assertEqual(arc, arc_copy)

        # SHAPES
        path = Path("M5,5V10Z")
        path_copy = copy(path)
        self.assertEqual(path, path_copy)
        self.assertIsNotNone(path_copy.values)

        rect = Rect(0, 0, 1000, 1000, ry=20)
        rect_copy = copy(rect)
        self.assertEqual(rect, rect_copy)
        self.assertIsNotNone(rect_copy.values)

        ellipse = Ellipse(0, 0, 1000, 1000)
        ellipse_copy = copy(ellipse)
        self.assertEqual(ellipse, ellipse_copy)
        self.assertIsNotNone(ellipse_copy.values)

        circle = Circle(x=0, y=0, r=1000)
        circle_copy = copy(circle)
        self.assertEqual(circle, circle_copy)
        self.assertIsNotNone(circle_copy.values)

        sline = SimpleLine((0, 0), (1000, 1000))
        sline_copy = copy(sline)
        self.assertEqual(sline, sline_copy)
        self.assertIsNotNone(sline_copy.values)

        rect = Rect(0, 0, 1000, 1000, ry=20)
        rect_copy = copy(rect)
        self.assertEqual(rect, rect_copy)
        self.assertIsNotNone(rect_copy.values)

        pline = Polyline((0, 0), (1000, 1000), (0,1000), (0,0))
        pline_copy = copy(pline)
        self.assertEqual(pline, pline_copy)
        self.assertIsNotNone(pline_copy.values)

        pgon = Polygon((0, 0), (1000, 1000), (0,1000))
        pgon_copy = copy(pgon)
        self.assertEqual(pgon, pgon_copy)
        self.assertIsNotNone(pgon_copy.values)

        group = Group(stroke="cornflower")
        group_copy = copy(group)
        self.assertEqual(group, group_copy)
        self.assertIsNotNone(group_copy.values)

        cpath = ClipPath(stroke="blue")
        cpath_copy = copy(cpath)
        self.assertEqual(cpath, cpath_copy)
        self.assertIsNotNone(cpath_copy.values)

        text = SVGText("HelloWorld")
        text_copy = copy(text)
        # self.assertEqual(text,text_copy)
        self.assertIsNotNone(text_copy.values)

        image = SVGImage(viewbox=viewbox)
        image_copy = copy(image)
        # self.assertEqual(image,image_copy)
        self.assertIsNotNone(image_copy.values)

        desc = Desc({"apple": 7}, desc="Some description")
        desc_copy = copy(desc)
        # self.assertEqual(desc, desc_copy)
        self.assertIsNotNone(desc_copy.values)

        title = Title({"apple": 3}, title="Some Title")
        title_copy = copy(title)
        # self.assertEqual(title, title_copy)
        self.assertIsNotNone(title_copy.values)

        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg>
                        <path d="M0 36c0 128.082 96 214 251 214c153.639 0 293 -37 293 -264c0 -220 -158.05 -325.976 -254 -391c-121 -82 -200.248 -117.634 -339 -171c-5.93907 -2.28437 -11.3166 -3.27496 -15.9795 -3.27496c-11.9348 0 -19.1871 6.48977 -19.1871 14.3854 c0 7.04844 5.77949 15.2172 19.1666 20.8896c118 50 225 86 316 200c91 113 125 205.913 125 350c0 135 -33 224 -145 224c-46.615 0 -77.452 -12.593 -112 -44c-8.09234 -7.35667 -11.3121 -13.0296 -11.3121 -17.4022c0 -13.4585 30.5027 -14.5978 43.3121 -14.5978 c69 0 123 -64.8867 123 -136c0 -75 -48 -132 -139 -132c-79.1582 0 -135 74 -135 150zM572 128c0 35 29 64 64 64s63 -29 63 -64s-28 -63 -63 -63s-64 28 -64 63zM572 -131c0 35 29 64 64 64s63 -29 63 -64s-28 -63 -63 -63s-64 28 -64 63z" transform="rotate(45deg)"/>
                        </svg>''')
        svg = SVG.parse(q)
        svg_copy = copy(svg)
        self.assertEqual(svg, svg_copy)
        self.assertIsNotNone(svg_copy.values)



import unittest
from random import *

from svgelements import *


def get_random_arc():
    return Arc((random() * 50, random() * 50),
               random() * 48 + 2, random() * 48 + 2,
               int(random() * 180),
               int(random() * 2), int(random() * 2),
               (random() * 50, random() * 50))


def get_random_circle_arc():
    r = random() * 48 + 2
    return Arc((random() * 50, random() * 50),
               r, r,
               int(random() * 180),
               int(random() * 2), int(random() * 2),
               (random() * 50, random() * 50))


class TestElementArcLength(unittest.TestCase):

    def test_arc_angle_point(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1)
            angle = random() * tau / 2 - tau / 4

            p = ellipse.point_at_angle(angle)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(angle, a)

    def test_arc_angle_point_rotated(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1, "rotate(45deg)")
            angle = random() * tau / 2 - tau / 4

            p = ellipse.point_at_angle(angle)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(angle, a)

    def test_arc_angles(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1)
            start = random() * tau / 2 - tau / 4
            end = random() * tau / 2 - tau / 4

            p = ellipse.point_at_angle(start)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(start, a)

            p = ellipse.point_at_angle(end)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(end, a)

            arc = ellipse.arc_angle(start, end)
            self.assertAlmostEqual(arc.get_start_angle(), start)
            self.assertAlmostEqual(arc.get_end_angle(), end)

    def test_arc_t(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1)
            start = random() * tau / 2 - tau / 4
            end = random() * tau / 2 - tau / 4

            p = ellipse.point_at_t(start)
            a = ellipse.t_at_point(p)
            self.assertAlmostEqual(start, a)

            p = ellipse.point_at_t(end)
            a = ellipse.t_at_point(p)
            self.assertAlmostEqual(end, a)

            arc = ellipse.arc_t(start, end)
            self.assertAlmostEqual(arc.get_start_t(), start)
            self.assertAlmostEqual(arc.get_end_t(), end)

    def test_arc_angles_rotated(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1, "rotate(90deg)")
            start = random() * tau / 2 - tau / 4
            end = random() * tau / 2 - tau / 4

            p = ellipse.point_at_angle(start)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(start, a)

            p = ellipse.point_at_t(end)
            a = ellipse.t_at_point(p)
            self.assertAlmostEqual(end, a)

            p = ellipse.point_at_angle(end)
            a = ellipse.angle_at_point(p)
            self.assertAlmostEqual(end, a)

            arc = ellipse.arc_angle(start, end)
            self.assertAlmostEqual(arc.get_start_angle(), start)
            self.assertAlmostEqual(arc.get_end_angle(), end)

    def test_arc_t_rotated(self):
        for i in range(1000):
            ellipse = Ellipse((0, 0), 2, 1,  "rotate(90deg)")
            start = random() * tau / 2 - tau / 4
            end = random() * tau / 2 - tau / 4

            p = ellipse.point_at_t(start)
            a = ellipse.t_at_point(p)
            self.assertAlmostEqual(start, a)

            arc = ellipse.arc_t(start, end)
            self.assertAlmostEqual(arc.get_start_t(), start)
            self.assertAlmostEqual(arc.get_end_t(), end)

    def test_arc_solve_produced(self):
        a = 3.05
        b = 2.23
        angle = atan(a * tan(radians(50)) / b)
        x = cos(angle) * a
        y = sin(angle) * b
        arc0 = Arc(start=3.05 + 0j, radius=3.05 + 2.23j, rotation=0, sweep_flag=1, arc_flag=0, end=x + 1j * y)

        ellipse = Ellipse(0, 0, 3.05, 2.23)
        arc1 = ellipse.arc_angle(0, Angle.degrees(50))

        self.assertEqual(arc0, arc1)

    def test_arc_solved_exact(self):
        ellipse = Ellipse(0.0, 0.0, 3.05, 2.23)
        arc = ellipse.arc_angle(0, Angle.degrees(50))
        arc *= "rotate(1)"
        exact = arc._exact_length()
        self.assertAlmostEqual(exact, 2.5314195265536624417, delta=1e-10)

    def test_arc_solved_integrated(self):
        ellipse = Ellipse(0, 0, 3.05, 2.23)
        arc = ellipse.arc_angle(0, Angle.degrees(50))
        length_calculated = arc._integral_length()
        self.assertAlmostEqual(length_calculated, 2.5314195265536624417, delta=1e-4)

    def test_arc_solved_lines(self):
        ellipse = Ellipse(0, 0, 3.05, 2.23)
        arc = ellipse.arc_angle(0, Angle.degrees(50))
        length_calculated = arc._line_length()
        self.assertAlmostEqual(length_calculated, 2.5314195265536624417, delta=1e-9)

    def test_arc_rotated_solved_exact(self):
        ellipse = Ellipse(0, 0, 3.05, 2.23)
        arc = ellipse.arc_angle(Angle.degrees(180), Angle.degrees(180 - 50))
        exact = arc._exact_length()
        self.assertAlmostEqual(exact, 2.5314195265536624417)

        arc = ellipse.arc_angle(Angle.degrees(360 + 180 - 50), Angle.degrees(180))
        exact = arc._exact_length()
        self.assertAlmostEqual(exact, 14.156360641292059)

    def test_arc_position_0_ortho(self):
        arc = Ellipse(0, 0, 3, 5).arc_angle(0, Angle.degrees(90))
        self.assertEqual(arc.point(0), (3, 0))

    def test_arc_position_0_rotate(self):
        arc = Ellipse(0, 0, 3, 5).arc_angle(0, Angle.degrees(90))
        arc *= "rotate(90deg)"
        p = arc.point(0)
        self.assertEqual(p, (0, 3))
        p = arc.point(1)
        self.assertEqual(p, (-5, 0))

    def test_arc_position_0_angle(self):
        arc = Ellipse("0,0", 3, 5).arc_angle(0, Angle.degrees(90))
        arc *= "rotate(-33deg)"
        self.assertEqual(arc.get_start_angle(), Angle.degrees(-33))

    def test_arc_position_0(self):
        start = Point(13.152548373912, 38.873772319489)
        arc = Arc(start,
                  Point(14.324014604836, 24.436855715076),
                  Point(-14.750000067599, 25.169681093411),
                  Point(-43.558410063178, 28.706909065029),
                  Point(-19.42967575562, -12.943218880396),
                  5.89788464227)
        point_0 = arc.point(0)
        self.assertAlmostEqual(start, point_0)

    def test_arc_len_r0_default(self):
        """Test error vs. random arc"""
        arc = Arc(Point(13.152548373912, 38.873772319489),
                  Point(14.324014604836, 24.436855715076),
                  Point(-14.750000067599, 25.169681093411),
                  Point(-43.558410063178, 28.706909065029),
                  Point(-19.42967575562, -12.943218880396),
                  5.89788464227)
        length = arc.length()
        self.assertAlmostEqual(198.3041678406902, length, places=3)

    def test_arc_len_r0_lines(self):
        """Test error vs. random arc"""
        arc = Arc(Point(13.152548373912, 38.873772319489),
                  Point(14.324014604836, 24.436855715076),
                  Point(-14.750000067599, 25.169681093411),
                  Point(-43.558410063178, 28.706909065029),
                  Point(-19.42967575562, -12.943218880396),
                  5.89788464227)
        length = arc._line_length()
        self.assertAlmostEqual(198.3041678406902, length, places=3)

    def test_arc_len_r0_exact(self):
        """Test error vs. random arc"""
        arc = Arc(Point(13.152548373912, 38.873772319489),
                  Point(14.324014604836, 24.436855715076),
                  Point(-14.750000067599, 25.169681093411),
                  Point(-43.558410063178, 28.706909065029),
                  Point(-19.42967575562, -12.943218880396),
                  5.89788464227)
        length = arc._exact_length()
        self.assertAlmostEqual(198.3041678406902, length, places=3)

    def test_arc_len_r0_integral(self):
        """Test error vs. random arc"""
        arc = Arc(Point(13.152548373912, 38.873772319489),
                  Point(14.324014604836, 24.436855715076),
                  Point(-14.750000067599, 25.169681093411),
                  Point(-43.558410063178, 28.706909065029),
                  Point(-19.42967575562, -12.943218880396),
                  5.89788464227)
        length = arc._integral_length()
        self.assertAlmostEqual(198.3041678406902, length, places=3)

    def test_arc_len_straight(self):
        """Test error at extreme eccentricities"""
        self.assertAlmostEqual(Arc(0, 1, 1e-10, 0, 1, 0, (0, 2e-10))._line_length(), 2, places=15)
        self.assertAlmostEqual(Arc(0, 1, 1e-10, 0, 1, 0, (0, 2e-10))._integral_length(), 2, places=5)
        self.assertEqual(Arc(0, 1, 1e-10, 0, 1, 0, (0, 2e-10))._exact_length(), 2)

    def test_unit_matrix(self):
        ellipse = Ellipse("20", "20", 4, 8, "rotate(45deg)")
        matrix = ellipse.unit_matrix()
        ellipse2 = Circle()
        ellipse2.values[SVG_ATTR_VECTOR_EFFECT] = SVG_VALUE_NON_SCALING_STROKE
        ellipse2 *= matrix
        p1 = ellipse.point_at_t(1)
        p2 = ellipse2.point_at_t(1)
        self.assertAlmostEqual(p1, p2)
        self.assertEqual(ellipse, ellipse2)

    def test_arc_len_circle_shortcut(self):
        """Known chord vs. shortcut"""
        error = 0
        for i in range(1000):
            arc = get_random_circle_arc()
            chord = abs(arc.sweep * arc.rx)
            length = arc.length()
            c = abs(length - chord)
            error += c
            self.assertAlmostEqual(chord, length)
        print("Average chord vs shortcut-length: %g" % (error / 1000))

    def test_arc_len_circle_int(self):
        """Known chord vs integral"""
        n = 10
        error = 0
        for i in range(n):
            arc = get_random_circle_arc()
            chord = abs(arc.sweep * arc.rx)
            length = arc._integral_length()
            c = abs(length - chord)
            error += c
            self.assertAlmostEqual(chord, length)
        print("Average chord vs integral: %g" % (error / n))

    def test_arc_len_circle_exact(self):
        """Known chord vs exact"""
        n = 1000
        error = 0
        for i in range(n):
            arc = get_random_circle_arc()
            chord = abs(arc.sweep * arc.rx)
            length = arc._exact_length()
            c = abs(length - chord)
            error += c
            self.assertAlmostEqual(chord, length)
        print("Average chord vs exact: %g" % (error / n))

    def test_arc_len_circle_line(self):
        """Known chord vs line"""
        n = 1
        error = 0
        for i in range(n):
            arc = get_random_circle_arc()
            chord = abs(arc.sweep * arc.rx)
            length = arc._line_length()
            c = abs(length - chord)
            error += c
            self.assertAlmostEqual(chord, length, places=6)
        print("Average chord vs line: %g" % (error / n))

    def test_arc_len_flat_line(self):
        """Known flat vs line"""
        n = 100
        error = 0
        for i in range(n):
            flat = 1 + random() * 50
            arc = Arc(0, flat, 1e-10, 0, 1, 0, (0, 2e-10))
            flat = 2*flat
            length = arc._line_length()
            c = abs(length - flat)
            error += c
            self.assertAlmostEqual(flat, length)
        print("Average flat vs line: %g" % (error / n))

    def test_arc_len_flat_integral(self):
        """Known flat vs integral"""
        n = 10
        error = 0
        for i in range(n):
            flat = 1 + random() * 50
            arc = Arc(0, flat, 1e-10, 0, 1, 0, (0, 2e-10))
            flat = 2*flat
            length = arc._integral_length()
            c = abs(length - flat)
            error += c
            self.assertAlmostEqual(flat, length)
        print("Average flat vs integral: %g" % (error / n))

    def test_arc_len_flat_exact(self):
        """Known flat vs exact"""
        n = 1000
        error = 0
        for i in range(n):
            flat = 1 + random() * 50
            arc = Arc(0, flat, 1e-10, 0, 1, 0, (0, 2e-10))
            flat = 2*flat
            length = arc._exact_length()
            c = abs(length - flat)
            error += c
            self.assertAlmostEqual(flat, length)
        print("Average flat vs line: %g" % (error / n))

    def test_arc_len_random_int(self):
        """Test error vs. random arc"""
        n = 5
        error = 0
        for i in range(n):
            arc = get_random_arc()
            length = arc._integral_length()
            exact = arc._exact_length()
            c = abs(length - exact)
            error += c
            self.assertAlmostEqual(exact, length, places=1)
        print("Average arc-integral error: %g" % (error / n))

    def test_arc_len_random_lines(self):
        """Test error vs. random arc"""
        n = 2
        error = 0
        for i in range(n):
            arc = get_random_arc()
            length = arc._line_length()
            exact = arc._exact_length()
            c = abs(length - exact)
            error += c
            self.assertAlmostEqual(exact, length, places=1)
        print("Average arc-line error: %g" % (error / n))

    def test_arc_issue_126(self):
        """
        Numerical Instability within arc bulge code.
        """
        arc = Arc(
            start=(-35.61856796405604, -3.1190066784519077),
            end=(-37.881309663852996, -5.381748378248861),
            bulge=0.9999999999999999
        )
        self.assertLessEqual(arc.sweep, tau/2)


class TestElementArcPoint(unittest.TestCase):

    def test_arc_point_start_stop(self):
        import numpy as np
        for _ in range(1000):
            arc = get_random_arc()
            self.assertEqual(arc.start, arc.point(0))
            self.assertEqual(arc.end, arc.point(1))
            self.assertTrue(np.all(np.array([list(arc.start), list(arc.end)])
                                   == arc.npoint([0, 1])))

    def test_arc_point_implementations_match(self):
        import numpy as np
        for _ in range(1000):
            arc = get_random_arc()

            pos = np.linspace(0, 1, 100)

            v1 = arc.npoint(pos)
            v2 = []
            for i in range(len(pos)):
                v2.append(arc.point(pos[i]))

            for p, p1, p2 in zip(pos, v1, v2):
                self.assertEqual(arc.point(p), Point(p1))
                self.assertEqual(Point(p1), Point(p2))


class TestElementArcApproximation(unittest.TestCase):

    def test_approx_quad(self):
        n = 100
        for i in range(n):
            arc = get_random_arc()
            path1 = Path([Move(), arc])
            path2 = Path(path1)
            path2.approximate_arcs_with_quads(error=0.05)
            d = abs(path1.length() - path2.length())
            # Error less than 1% typically less than 0.5%
            if d > 10:
                print(arc)
            self.assertAlmostEqual(d, 0.0, delta=20)

    def test_approx_cubic(self):
        n = 100
        for i in range(n):
            arc = get_random_arc()
            path1 = Path([Move(), arc])
            path2 = Path(path1)
            path2.approximate_arcs_with_cubics(error=0.1)
            d = abs(path1.length() - path2.length())
            # Error less than 0.1% typically less than 0.001%
            if d > 1:
                print(arc)
            self.assertAlmostEqual(d, 0.0, delta=2)

    def test_approx_quad_degenerate(self):
        arc = Arc(start=(0,0),end=(0,0), control=(0,0))
        path1 = Path([Move(), arc])
        path2 = Path(path1)
        path2.approximate_arcs_with_quads(error=0.05)
        d = abs(path1.length() - path2.length())
        # Error less than 1% typically less than 0.5%
        if d > 10:
            print(arc)
        self.assertAlmostEqual(d, 0.0, delta=20)

    def test_approx_cubic_degenerate(self):
        arc = Arc(start=(0,0),end=(0,0), control=(0,0))
        path1 = Path([Move(), arc])
        path2 = Path(path1)
        path2.approximate_arcs_with_cubics(error=0.1)
        d = abs(path1.length() - path2.length())
        # Error less than 0.1% typically less than 0.001%
        if d > 1:
            print(arc)
        self.assertAlmostEqual(d, 0.0, delta=2)


import io
import unittest

from svgelements import *


class TestElementGroup(unittest.TestCase):

    def test_group_bbox(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg viewBox="0, 0, 100, 100" height="200" width="200">
                        <g>
                        <rect x="0" y="20" width="50" height="50"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q, width=500, height=500)
        m *= 'scale(2)'
        for e in m.select(lambda e: isinstance(e, Rect)):
            self.assertEqual(e.x, 0)
            self.assertEqual(e.y, 40)
            self.assertEqual(e.width, 100)
            self.assertEqual(e.height, 100)
        self.assertEqual(m.width, 200)
        self.assertEqual(m.height, 200)

    def test_group_2rect(self):
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg viewBox="0, 0, 100, 100" height="200" width="200">
                        <g>
                        <rect x="0" y="20" width="50" height="50"/>
                        <rect x="0" y="0" width="50" height="50" transform="rotate(45)"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q, width=500, height=500, reify=False)
        m *= 'scale(2)'
        rects = list(m.select(lambda e: isinstance(e, Rect)))
        r0 = rects[0]
        self.assertEqual(r0.implicit_x, 0)
        self.assertEqual(r0.implicit_y, 80)
        self.assertEqual(r0.implicit_width, 200)
        self.assertEqual(r0.implicit_height, 200)
        self.assertEqual(m.width, 200)
        self.assertEqual(m.height, 200)
        self.assertEqual(r0.bbox(), (0.0, 80.0, 200.0, 280.0))
        m.reify()
        self.assertEqual(m.implicit_width, 400)
        self.assertEqual(m.implicit_height, 400)
        r1 = rects[1]
        self.assertEqual(r1.implicit_x, 0)
        self.assertEqual(r1.implicit_y, 0)
        self.assertAlmostEqual(r1.implicit_width, 200)
        self.assertAlmostEqual(r1.implicit_height, 200)
        print(r1.bbox())

    def test_issue_107(self):
        """
        Tests issue 107 inability to multiple group matrix objects while creating new group objects.

        https://github.com/meerk40t/svgelements/issues/107
        """
        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
                        <svg viewBox="0, 0, 100, 100" height="200" width="200">
                        <g>
                        <rect x="0" y="20" width="50" height="50"/>
                        </g>
                        </svg>''')
        m = SVG.parse(q)
        m *= "translate(100,100)"  # Test __imul__
        n = m * 'scale(2)'  # Test __mult__
        self.assertEqual(n[0][0].transform, Matrix("matrix(2,0,0,2,200,200)"))
        self.assertEqual(m[0][0].transform, Matrix("matrix(1,0,0,1,100,100)"))

    def test_issue_152(self):
        """
        Tests issue 152, closed text objects within a group with style:display=None
        This should have the SVG element and nothing else.

        https://github.com/meerk40t/svgelements/issues/152
        """
        q = io.StringIO(u'''<?xml version="1.0" encoding="UTF-8" standalone="no"?><svg>
        <g style="display:none">
            <text><textPath><tspan>Issue 152</tspan></textPath></text>
        </g>
        </svg>''')
        elements = list(SVG.parse(q).elements())
        self.assertEqual(len(elements), 1)
